from dataclasses import dataclass, field
from typing import Dict, Optional


@dataclass
class ScheduleConfig:
    epochs: int = 3
    ema: Optional[Dict] = field(default=None)
    optimizer: Dict = field(default_factory=lambda: {
        "name": "adamw",
        "lr": 6e-5,
        "betas": [0.9, 0.999],
        "weight_decay": 0.0005,
    })
    scheduler: Dict = field(default_factory=lambda: {
        "name": "cosine_no_sgdr",
        "warmup_epochs": 5,
        "warmup_bias_lr": 1e-5,
        "min_lr": 0.,
    })


@dataclass
class ClassificationScheduleConfig(ScheduleConfig):
    pass


@dataclass
class SegmentationScheduleConfig(ScheduleConfig):
    pass


@dataclass
class DetectionScheduleConfig(ScheduleConfig):
    pass



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, ModelConfig


@dataclass
class SegFormerB0ArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mixtransformer",
            "params": {
                "ffn_intermediate_expansion_ratio": 4,
                "ffn_act_type": "gelu",
                "ffn_dropout_prob": 0.0,
                "attention_dropout_prob": 0.0,
            },
            "stage_params": [
                {
                    "num_blocks": 2,
                    "sequence_reduction_ratio": 8,
                    "attention_chananels": 32,
                    "embedding_patch_sizes": 7,
                    "embedding_strides": 4,
                    "num_attention_heads": 1,
                },
                {
                    "num_blocks": 2,
                    "sequence_reduction_ratio": 4,
                    "attention_chananels": 64,
                    "embedding_patch_sizes": 3,
                    "embedding_strides": 2,
                    "num_attention_heads": 2,
                },
                {
                    "num_blocks": 2,
                    "sequence_reduction_ratio": 2,
                    "attention_chananels": 160,
                    "embedding_patch_sizes": 3,
                    "embedding_strides": 2,
                    "num_attention_heads": 5,
                },
                {
                    "num_blocks": 2,
                    "sequence_reduction_ratio": 1,
                    "attention_chananels": 256,
                    "embedding_patch_sizes": 3,
                    "embedding_strides": 2,
                    "num_attention_heads": 8,
                },
            ],
        }
    )


@dataclass
class SegmentationSegFormerB0ModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "segformer_b0"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: SegFormerB0ArchitectureConfig(
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                    "resize_output": [512, 512],
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )



from pathlib import Path

from requests_toolbelt.multipart.encoder import MultipartEncoder
from tqdm import tqdm

from netspresso.clients.utils.system import ENV_STR

version = (Path(__file__).parent.parent.parent / "VERSION").read_text().strip()


def get_headers(access_token=None, json_type=False):
    headers = {"User-Agent": f"NetsPresso Python Package v{version} ({ENV_STR})"}
    if access_token:
        headers["Authorization"] = f"Bearer {access_token}"
    if json_type:
        headers["Content-Type"] = "application/json"
    return headers


def create_tao_headers(token):
    return {"Authorization": f"Bearer {token}"}


def read_file_bytes(file_path: str) -> bytes:
    with open(file_path, "rb") as f:
        file_byte = f.read()
    return file_byte


def get_files(file_path):
    return [
        (
            "file",
            (Path(file_path).name, open(file_path, "rb"), "application/octet-stream"),
        )
    ]


def create_multipart_data(url, file_info):
    # Prepare the multipart form data
    file_name = file_info[0]
    file_content = file_info[1]
    multipart_data = MultipartEncoder(
            fields={
                "url": (None, url, "application/json"),
                "file": (file_name, file_content, "application/octet-stream"),
            }
        )

    return multipart_data


def create_progress_func(multipart_data):
    # Progress callback function
    progress = tqdm(
        total=multipart_data.len,
        unit="B",
        unit_scale=True,
        unit_divisor=1024,
        colour="#1BBFD6",
        desc="Uploading model",
    )

    return progress


def progress_callback(monitor, progress):
    progress.update(monitor.bytes_read - progress.n)



from dataclasses import dataclass

from netspresso.enums import TaskStatusForDisplay


@dataclass
class Device:
    device_brand: str
    device_name: str


@dataclass
class TaskStatusInfo:
    status: TaskStatusForDisplay



from dataclasses import asdict, dataclass
from typing import Dict


@dataclass
class BaseScheduler:
    def asdict(self) -> Dict:
        return asdict(self)


@dataclass
class StepLR(BaseScheduler):
    name: str = "step"
    iters_per_phase: int = 1
    gamma: float = 0.1


@dataclass
class PolynomialLRWithWarmUp(BaseScheduler):
    name: str = "poly"
    warmup_epochs: int = 5
    warmup_bias_lr: float = 1e-5
    min_lr: float = 1e-6
    power: float = 1.0


@dataclass
class CosineAnnealingLRWithCustomWarmUp(BaseScheduler):
    name: str = "cosine_no_sgdr"
    warmup_epochs: int = 5
    warmup_bias_lr: float = 1e-5
    min_lr: float = 1e-6


@dataclass
class CosineAnnealingWarmRestartsWithCustomWarmUp(BaseScheduler):
    name: str = "cosine"
    warmup_epochs: int = 5
    warmup_bias_lr: float = 1e-5
    min_lr: float = 1e-6
    iters_per_phase: int = 10



from enum import Enum

from aenum import NamedConstant


class Task(str, Enum):
    IMAGE_CLASSIFICATION = "classification"
    OBJECT_DETECTION = "detection"
    SEMANTIC_SEGMENTATION = "segmentation"


class LauncherTask(str, Enum):
    CONVERT = "convert"
    BENCHMARK = "benchmark"


class TaskStatusForDisplay(NamedConstant):
    # task status for display - launchx
    IN_QUEUE = "IN_QUEUE"
    IN_PROGRESS = "IN_PROGRESS"
    FINISHED = "FINISHED"
    ERROR = "ERROR"
    USER_CANCEL = "USER_CANCEL"
    TIMEOUT = "TIMEOUT"



import warnings
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from loguru import logger
from omegaconf import OmegaConf

from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.launcher import launcher_client_v2
from netspresso.enums import Framework, Optimizer, Scheduler, ServiceTask, Status, Task
from netspresso.exceptions.trainer import (
    BaseDirectoryNotFoundException,
    DirectoryNotFoundException,
    FailedTrainingException,
    FileNotFoundErrorException,
    NotSetDatasetException,
    NotSetModelException,
    NotSupportedModelException,
    NotSupportedTaskException,
    RetrainingFunctionException,
    TaskOrYamlPathException,
)
from netspresso.metadata.common import InputShape
from netspresso.metadata.trainer import TrainerMetadata
from netspresso.trainer.augmentations import AUGMENTATION_CONFIG_TYPE, AugmentationConfig, Transform
from netspresso.trainer.data import DATA_CONFIG_TYPE, ImageLabelPathConfig, PathConfig
from netspresso.trainer.models import (
    CLASSIFICATION_MODELS,
    DETECTION_MODELS,
    SEGMENTATION_MODELS,
    CheckpointConfig,
    ModelConfig,
)
from netspresso.trainer.trainer_configs import TrainerConfigs
from netspresso.trainer.training import TRAINING_CONFIG_TYPE, EnvironmentConfig, LoggingConfig, ScheduleConfig
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class Trainer(NetsPressoBase):
    def __init__(
        self, token_handler: TokenHandler, task: Optional[Union[str, Task]] = None, yaml_path: Optional[str] = None
    ) -> None:
        """Initialize the Trainer.

        Args:
            task (Union[str, Task]], optional): The type of task (classification, detection, segmentation). Either 'task' or 'yaml_path' must be provided, but not both.
            yaml_path (str, optional): Path to the YAML configuration file. Either 'task' or 'yaml_path' must be provided, but not both.
        """

        self.token_handler = token_handler
        self.deprecated_names = {
            "EfficientFormer": "EfficientFormer-L1",
            "MobileNetV3_Small": "MobileNetV3-S",
            "MobileNetV3_Large": "MobileNetV3-L",
            "ViT-Tiny": "ViT-T",
            "MixNet-Small": "MixNet-S",
            "MixNet-Medium": "MixNet-M",
            "MixNet-Large": "MixNet-L",
            "PIDNet": "PIDNet-S",
        }

        if (task is not None) == (yaml_path is not None):
            raise TaskOrYamlPathException()

        if task is not None:
            self._initialize_from_task(task)
        elif yaml_path is not None:
            self._initialize_from_yaml(yaml_path)

    def _initialize_from_task(self, task: Union[str, Task]) -> None:
        """Initialize the Trainer object based on the provided task.

        Args:
            task (Union[str, Task]): The task for which the Trainer is initialized.
        """

        self.task = self._validate_task(task)
        self.available_models = list(self._get_available_models().keys())
        self.data = None
        self.model = None
        self.training = TRAINING_CONFIG_TYPE[self.task]()
        self.augmentation = AUGMENTATION_CONFIG_TYPE[self.task]()
        self.logging = LoggingConfig()
        self.environment = EnvironmentConfig()

    def _initialize_from_yaml(self, yaml_path: str) -> None:
        """Initialize the Trainer object based on the configuration provided in a YAML file.

        Args:
            yaml_path (str): The path to the YAML file containing the configuration.
        """

        hparams = OmegaConf.load(yaml_path)
        hparams["model"].pop("single_task_model")

        metadata_path = Path(yaml_path).parent / "metadata.json"
        metadata = FileHandler.load_json(metadata_path)
        self.model_name = metadata["model_info"]["model"]

        self.img_size = hparams["augmentation"]["img_size"]
        self.task = hparams["data"]["task"]
        self.available_models = list(self._get_available_models().keys())

        self.data = DATA_CONFIG_TYPE[self.task](**hparams["data"])
        self.model = ModelConfig(**hparams["model"])
        self.training = ScheduleConfig(**hparams["training"])
        self.augmentation = AugmentationConfig(**hparams["augmentation"])
        self.logging = LoggingConfig(**hparams["logging"])
        self.environment = EnvironmentConfig(**hparams["environment"])

    def _validate_task(self, task: Union[str, Task]):
        """Validate the provided task.

        Args:
            task (Union[str, Task]): The task to be validated.

        Raises:
            ValueError: If the provided task is not supported.

        Returns:
            Task: The validated task.
        """

        available_tasks = [task.value for task in Task]
        if task not in available_tasks:
            raise NotSupportedTaskException(available_tasks, task)
        return task

    def _validate_config(self):
        """Validate the configuration setup.

        Raises:
            ValueError: Raised if the dataset is not set. Use `set_dataset_config` or `set_dataset_config_with_yaml` to set the dataset configuration.
            ValueError: Raised if the model is not set. Use `set_model_config` or `set_model_config_with_yaml` to set the model configuration.
        """

        if self.data is None:
            raise NotSetDatasetException()
        if self.model is None:
            raise NotSetModelException()

    def _get_available_models(self) -> Dict[str, Any]:
        """Get available models based on the current task.

        Returns:
            Dict[str, Any]: A dictionary mapping model types to available models.
        """

        available_models = {
            "classification": CLASSIFICATION_MODELS,
            "detection": DETECTION_MODELS,
            "segmentation": SEGMENTATION_MODELS,
        }[self.task]

        # Filter out deprecated names
        filtered_models = {
            name: config
            for name, config in available_models.items()
            if name not in self.deprecated_names
        }

        return filtered_models

    def _get_available_models_w_deprecated_names(self) -> Dict[str, Any]:
        """Get available models based on the current task.

        Returns:
            Dict[str, Any]: A dictionary mapping model types to available models.
        """

        available_models = {
            "classification": CLASSIFICATION_MODELS,
            "detection": DETECTION_MODELS,
            "segmentation": SEGMENTATION_MODELS,
        }[self.task]

        return available_models

    def set_dataset_config(
        self,
        name: str,
        root_path: str,
        train_image: str = "images/train",
        train_label: str = "labels/train",
        valid_image: str = "images/valid",
        valid_label: str = "labels/valid",
        id_mapping: Optional[Union[List[str], Dict[str, str], str]] = None,
    ):
        """Set the dataset configuration for the Trainer.

        Args:
            name (str): The name of dataset.
            root_path (str): Root directory of dataset.
            train_image (str, optional): The directory for training images. Should be relative path to root directory. Defaults to "images/train".
            train_label (str, optional): The directory for training labels. Should be relative path to root directory. Defaults to "labels/train".
            valid_image (str, optional): The directory for validation images. Should be relative path to root directory. Defaults to "images/val".
            valid_label (str, optional): The directory for validation labels. Should be relative path to root directory. Defaults to "labels/val".
            id_mapping (Union[List[str], Dict[str, str]], optional): ID mapping for the dataset. Defaults to None.
        """

        common_config = {
            "name": name,
            "path": PathConfig(
                root=root_path,
                train=ImageLabelPathConfig(image=train_image, label=train_label),
                valid=ImageLabelPathConfig(image=valid_image, label=valid_label),
            ),
            "id_mapping": id_mapping,
        }
        self.data = DATA_CONFIG_TYPE[self.task](**common_config)

    def check_paths_exist(self, base_path):
        paths = [
            "images/train",
            "images/valid",
            "id_mapping.json",
        ]

        # Check for the existence of required directories and files
        for relative_path in paths:
            path = Path(base_path) / relative_path
            if not path.exists():
                if path.suffix:  # It's a file
                    raise FileNotFoundErrorException(relative_path)
                else:  # It's a directory
                    raise DirectoryNotFoundException(relative_path)

    def find_paths(self, base_path: str, search_dir, split: str) -> List[str]:
        base_dir = Path(base_path)

        if not base_dir.exists():
            raise BaseDirectoryNotFoundException(base_dir)

        result_paths = []

        dir_path = base_dir / search_dir
        if dir_path.exists() and dir_path.is_dir():
            for item in dir_path.iterdir():
                if (item.is_dir() or item.is_file()) and split in item.name:
                    result_paths.append(item.as_posix())

        return result_paths[0]

    def set_dataset(self, dataset_root_path: str):
        dataset_name = Path(dataset_root_path).name
        root_path = Path(dataset_root_path).resolve().as_posix()

        self.check_paths_exist(root_path)
        images_train = self.find_paths(root_path, "images", "train")
        images_valid = self.find_paths(root_path, "images", "valid")
        labels_train = self.find_paths(root_path, "labels", "train")
        labels_valid = self.find_paths(root_path, "labels", "valid")
        id_mapping = FileHandler.load_json(f"{root_path}/id_mapping.json")
        self.set_dataset_config(
            name=dataset_name,
            root_path=dataset_root_path,
            train_image=images_train,
            train_label=labels_train,
            valid_image=images_valid,
            valid_label=labels_valid,
            id_mapping=id_mapping,
        )

    def set_model_config(
        self,
        model_name: str,
        img_size: int,
        use_pretrained: bool = True,
        load_head: bool = False,
        path: Optional[str] = None,
        fx_model_path: Optional[str] = None,
        optimizer_path: Optional[str] = None,
    ):
        """Set the model configuration for the Trainer.

        Args:
            model_name (str): Name of the model.
            img_size (int): Image size for the model.
            use_pretrained (bool, optional): Whether to use a pre-trained model. Defaults to True.
            load_head (bool, optional): Whether to load the model head. Defaults to False.
            path (str, optional): Path to the model. Defaults to None.
            fx_model_path (str, optional): Path to the FX model. Defaults to None.
            optimizer_path (str, optional): Path to the optimizer. Defaults to None.

        Raises:
            ValueError: If the specified model is not supported for the current task.
        """

        if model_name in self.deprecated_names:
            warnings.filterwarnings("default", category=DeprecationWarning)
            warnings.warn(
                f"The model name '{model_name}' is deprecated and will be removed in a future version. "
                f"Please use '{self.deprecated_names[model_name]}' instead.",
                DeprecationWarning,
                stacklevel=2,
            )
            warnings.filterwarnings("ignore", category=DeprecationWarning)

        self.model_name = model_name
        model = self._get_available_models_w_deprecated_names().get(model_name)
        self.img_size = img_size
        self.logging.onnx_input_size = [img_size, img_size]

        if model is None:
            raise NotSupportedModelException()

        self.model = model(
            checkpoint=CheckpointConfig(
                use_pretrained=use_pretrained,
                load_head=load_head,
                path=path,
                fx_model_path=fx_model_path,
                optimizer_path=optimizer_path,
            )
        )

    def set_fx_model(self, fx_model_path: str):
        """Set the FX model path for retraining.

        Args:
            fx_model_path (str): The path to the FX model.

        Raises:
            ValueError: If the model is not set. Please use 'set_model_config' for model setup.
        """

        if not self.model:
            raise RetrainingFunctionException()

        self.model.checkpoint.path = None
        self.model.checkpoint.fx_model_path = fx_model_path

    def set_training_config(
        self,
        optimizer,
        scheduler,
        epochs: int = 3,
        batch_size: int = 8,
    ):
        """Set the training configuration.

        Args:
            optimizer: The configuration of optimizer.
            scheduler: The configuration of learning rate scheduler.
            epochs (int, optional): The total number of epoch for training the model. Defaults to 3.
            batch_size (int, optional): The number of samples in single batch input. Defaults to 8.
        """

        self.training = ScheduleConfig(
            epochs=epochs,
            optimizer=optimizer.asdict(),
            scheduler=scheduler.asdict(),
        )
        self.environment.batch_size = batch_size

    def set_augmentation_config(
        self,
        train_transforms: Optional[List] = None,
        inference_transforms: Optional[List] = None,
    ):
        """Set the augmentation configuration for training.

        Args:
            train_transforms (List, optional): List of transforms for training. Defaults to None.
            inference_transforms (List, optional): List of transforms for inference. Defaults to None.
        """

        self.augmentation = AugmentationConfig(
            train=train_transforms,
            inference=inference_transforms,
        )

    def set_logging_config(
        self,
        project_id: Optional[str] = None,
        output_dir: str = "./outputs",
        tensorboard: bool = True,
        csv: bool = False,
        image: bool = True,
        stdout: bool = True,
        save_optimizer_state: bool = True,
        validation_epoch: int = 10,
        save_checkpoint_epoch: Optional[int] = None,
    ):
        """Set the logging configuration.

        Args:
            project_id (str, optional): Project name to save the experiment. If None, it is set as {task}_{model} (e.g. segmentation_segformer).
            output_dir (str, optional): Root directory for saving the experiment. Defaults to "./outputs".
            tensorboard (bool, optional): Whether to use the tensorboard. Defaults to True.
            csv (bool, optional): Whether to save the result in csv format. Defaults to False.
            image (bool, optional): Whether to save the validation results. It is ignored if the task is classification. Defaults to True.
            stdout (bool, optional): Whether to log the standard output. Defaults to True.
            save_optimizer_state (bool, optional): Whether to save optimizer state with model checkpoint to resume training. Defaults to True.
            validation_epoch (int, optional): Validation frequency in total training process. Defaults to 10.
            save_checkpoint_epoch (int, optional): Checkpoint saving frequency in total training process. Defaults to None.
        """

        self.logging = LoggingConfig(
            project_id=project_id,
            output_dir=output_dir,
            tensorboard=tensorboard,
            csv=csv,
            image=image,
            stdout=stdout,
            save_optimizer_state=save_optimizer_state,
            validation_epoch=validation_epoch,
            save_checkpoint_epoch=save_checkpoint_epoch,
        )

    def set_environment_config(self, seed: int = 1, num_workers: int = 4):
        """Set the environment configuration.

        Args:
            seed (int, optional): Random seed. Defaults to 1.
            num_workers (int, optional): The number of multi-processing workers to be used by the data loader. Defaults to 4.
        """

        self.environment = EnvironmentConfig(seed=seed, num_workers=num_workers)

    def _change_transforms(self, transforms: Transform):
        """Update the 'size' attribute in the given list of transforms with the specified image size.

        Args:
            transforms (List[Transform]): The list of transforms to be updated.

        Returns:
            List[Transform]: The list of transforms with the 'size' attribute updated to the specified image size.
        """

        field_name_to_check = "size"

        if transforms is None:
            return transforms

        for transform in transforms:
            field_type = transform.__annotations__.get(field_name_to_check)

            if field_type == List:
                transform.size = [self.img_size, self.img_size]
            elif isinstance(field_type, int):
                transform.size = self.img_size

        return transforms

    def _apply_img_size(self):
        """Apply the specified image size to the augmentation configurations.

        This method updates the 'img_size' attribute in the augmentation configurations, including
        'train.transforms', 'train.mix_transforms', and 'inference.transforms'.
        """

        self.augmentation.img_size = self.img_size
        self.augmentation.train = self._change_transforms(self.augmentation.train)
        self.augmentation.inference = self._change_transforms(self.augmentation.inference)

    def _get_available_options(self):
        self.token_handler.validate_token()
        options_response = launcher_client_v2.converter.read_framework_options(
            access_token=self.token_handler.tokens.access_token,
            framework=Framework.ONNX,
        )

        available_options = options_response.data

        # TODO: Will be removed when we support DLC in the future
        available_options = [
            available_option
            for available_option in available_options
            if available_option.framework != "dlc"
        ]

        return available_options

    def _get_status_by_training_summary(self, status):
        status_mapping = {
            "success": Status.COMPLETED,
            "stop": Status.STOPPED,
            "error": Status.ERROR,
            "": Status.IN_PROGRESS
        }
        return status_mapping.get(status, Status.IN_PROGRESS)

    def initialize_metadata(self, output_dir):
        def create_metadata_with_status(status, error_message=None):
            metadata = TrainerMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = TrainerMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Training task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.update_output_dir(output_dir.resolve().as_posix())
            metadata.update_model_info(
                task=self.task,
                model=self.model_name,
                dataset=self.data.name,
                input_shapes=[InputShape(batch=1, channel=3, dimension=[self.img_size, self.img_size])],
            )
            metadata.update_training_info(
                epochs=self.training.epochs,
                batch_size=self.environment.batch_size,
                learning_rate=self.training.optimizer["lr"],
                optimizer=Optimizer.to_display_name(self.training.optimizer["name"]),
                scheduler=Scheduler.to_display_name(self.training.scheduler["name"]),
            )
            metadata.update_hparams(hparams=(output_dir / "hparams.yaml").resolve().as_posix())
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def find_best_model_paths(self, destination_folder: Path):
        best_fx_paths_set = set()

        for pattern in ["*best_fx.pt", "*best.pt"]:
            best_fx_paths_set.update(destination_folder.glob(pattern))

        best_fx_paths = list(best_fx_paths_set)
        best_onnx_paths = list(destination_folder.glob("*best.onnx"))

        return best_fx_paths, best_onnx_paths

    def create_runtime_config(self, yaml_path):
        hparams = OmegaConf.load(yaml_path)

        preprocess = hparams.augmentation.inference
        for _preprocess in preprocess:
            if _preprocess.size:
                _preprocess.size = _preprocess.size[0]
            if _preprocess.name == "resize":
                _preprocess.resize_criteria = "long"

        if hparams.model.task == Task.IMAGE_CLASSIFICATION:
            visualize = {"params": {"class_map": hparams.data.id_mapping, "pallete": None}}
        elif hparams.model.task == Task.OBJECT_DETECTION:
            visualize = {"params": {"class_map": hparams.data.id_mapping, "normalized": False, "brightness_factor": 1.5}}
        elif hparams.model.task == Task.SEMANTIC_SEGMENTATION:
            visualize = {"params": {"class_map": hparams.data.id_mapping, "pallete": None, "normalized": False, "brightness_factor": 1.5}}

        _config = {
            "task": hparams.model.task,
            "preprocess": preprocess,
            "postprocess": hparams.model.postprocessor,
            "visualize": visualize,
        }
        name = "runtime"
        config = OmegaConf.create({name: _config})
        save_path = Path(yaml_path).parent / f"{name}.yaml"
        OmegaConf.save(config=config, f=save_path)

        return save_path

    def train(self, gpus: str, project_name: str, output_dir: Optional[str] = "./outputs") -> TrainerMetadata:
        """Train the model with the specified configuration.

        Args:
            gpus (str): GPU ids to use, separated by commas.
            project_name (str): Project name to save the experiment.

        Returns:
            Dict: A dictionary containing information about the training.
        """

        from netspresso_trainer import train_with_yaml

        self._validate_config()
        self._apply_img_size()

        project_name = project_name if project_name else f"{self.task}_{self.model_name}".lower()
        destination_folder = Path(output_dir) / project_name
        destination_folder = FileHandler.create_unique_folder(folder_path=destination_folder)
        metadata = self.initialize_metadata(output_dir=destination_folder)

        try:
            self.logging.output_dir = output_dir
            self.logging.project_id = destination_folder.name
            self.logging_dir = Path(self.logging.output_dir) / self.logging.project_id / "version_0"
            self.environment.gpus = gpus

            configs = TrainerConfigs(
                self.data,
                self.augmentation,
                self.model,
                self.training,
                self.logging,
                self.environment,
            )
            train_with_yaml(
                gpus=gpus,
                data=configs.data,
                augmentation=configs.augmentation,
                model=configs.model,
                training=configs.training,
                logging=configs.logging,
                environment=configs.environment,
            )

            available_options = self._get_available_options()
            metadata.update_available_options(available_options)

        except Exception as e:
            e = FailedTrainingException(error_log=e.args[0])
            metadata = self.handle_error(metadata, ServiceTask.TRAINING, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.TRAINING)
        finally:
            FileHandler.remove_folder(configs.temp_folder)
            logger.info(f"Removed {configs.temp_folder} folder.")

            FileHandler.move_and_cleanup_folders(source_folder=self.logging_dir, destination_folder=destination_folder)
            logger.info(f"Files in {self.logging_dir} were moved to {destination_folder}.")

            runtime_config_path = self.create_runtime_config(yaml_path=destination_folder / "hparams.yaml")
            metadata.runtime = runtime_config_path.as_posix()

            training_summary = FileHandler.load_json(file_path=destination_folder / "training_summary.json")
            metadata.update_training_result(training_summary=training_summary)

            status = self._get_status_by_training_summary(training_summary.get("status"))
            metadata.update_status(status=status)
            if status == Status.ERROR:
                error_stats = training_summary.get("error_stats", "")
                e = FailedTrainingException(error_log=error_stats)
                metadata.update_message(exception_detail=e.args[0])

            best_fx_paths, best_onnx_paths = self.find_best_model_paths(destination_folder)
            if best_fx_paths:
                metadata.update_best_fx_model_path(best_fx_model_path=best_fx_paths[0].resolve().as_posix())
            if best_onnx_paths:
                metadata.update_best_onnx_model_path(best_onnx_model_path=best_onnx_paths[0].resolve().as_posix())
            MetadataHandler.save_metadata(data=metadata, folder_path=destination_folder)

        return metadata



from netspresso.clients.config import Config, ServiceModule, ServiceName
from netspresso.clients.launcher.v2.benchmarker import Benchmarker
from netspresso.clients.launcher.v2.converter import Converter


class LauncherAPIClient:
    def __init__(self):
        self.config = Config(ServiceName.NP, ServiceModule.LAUNCHER)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.prefix = self.config.URI_PREFIX
        self.url = f"{self.host}:{self.port}{self.prefix}"
        self.converter = self._create_convert_api()
        self.benchmarker = self._create_benchmark_api()

    def _create_convert_api(self):
        return Converter(self.url)

    def _create_benchmark_api(self):
        return Benchmarker(self.url)

    def is_cloud(self) -> bool:
        return self.config.is_cloud()


launcher_client_v2 = LauncherAPIClient()



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class ShufflenetV2ArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "shufflenetv2",
            "params": {
                "model_size": "0.5x"
            },
            "stage_params": None,
        }
    )


@dataclass
class DetectionYoloFastestModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "yolofastest"
    checkpoint: CheckpointConfig = field(default_factory=lambda: CheckpointConfig(use_pretrained=False))
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ShufflenetV2ArchitectureConfig(
            neck={
                "name": "lightfpn",
                "params": {
                    "out_channels": 72,
                },
            },
            head={
                "name": "yolo_fastest_head_v2",
                "params": {
                    "anchors": [
                        [12,18, 37,49, 52,132],  # P2
                        [115,73, 119,199, 242,238],  # P3
                    ]
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "retinanet_loss", "weight": None}]
    )



from typing import List

from netspresso.exceptions.common import AdditionalData, PyNPException


class NotSupportedTaskException(PyNPException):
    def __init__(self, available_tasks: List, task: int):
        message = f"The task supports {available_tasks}. The entered task is {task}."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class NotSetDatasetException(PyNPException):
    def __init__(self):
        message = "The dataset is not set. Use `set_dataset_config` or `set_dataset_config_with_yaml` to set the dataset configuration."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class NotSetModelException(PyNPException):
    def __init__(self):
        message = "The model is not set. Use `set_model_config` or `set_model_config_with_yaml` to set the model configuration."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )

class TaskOrYamlPathException(PyNPException):
    def __init__(self):
        message = "Either 'task' or 'yaml_path' must be provided, but not both."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class NotSupportedModelException(PyNPException):
    def __init__(self, available_models: List, model_name: str, task: str):
        message = f"The '{model_name}' model is not supported for the '{task}' task. The available models are {available_models}."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class RetrainingFunctionException(PyNPException):
    def __init__(self):
        message = "This function is intended for retraining. Please use 'set_model_config' for model setup."
        super().__init__(
                data=AdditionalData(origin="pynp"),
                error_code="",
                name=self.__class__.__name__,
                message=message,
            )


class FileNotFoundErrorException(PyNPException):
    def __init__(self, relative_path: str):
        message = f"The required file '{relative_path}' does not exist. Please check and make sure it is in the correct location."
        super().__init__(
                data=AdditionalData(origin="pynp"),
                error_code="",
                name=self.__class__.__name__,
                message=message,
            )


class DirectoryNotFoundException(PyNPException):
    def __init__(self, relative_path: str):
        message = f"The required directory '{relative_path}' does not exist. Please check and make sure it is in the correct location."
        super().__init__(
                data=AdditionalData(origin="pynp"),
                error_code="",
                name=self.__class__.__name__,
                message=message,
            )


class BaseDirectoryNotFoundException(PyNPException):
    def __init__(self, base_path: str):
        message = f"The directory '{base_path}' does not exist."
        super().__init__(
                data=AdditionalData(origin="pynp"),
                error_code="",
                name=self.__class__.__name__,
                message=message,
            )


class FailedTrainingException(PyNPException):
    def __init__(self, error_log: str):
        message = "An error occurred during the training process."
        super().__init__(
                data=AdditionalData(
                    origin="pynp",
                    error_log=error_log
                ),
                error_code="",
                name=self.__class__.__name__,
                message=message,
            )




from .base import InputLayer, ModelBase, ModelStatus
from .request_body import (
    RequestModelUploadUrl,
    RequestUploadModel,
    RequestValidateModel,
)
from .response_body import (
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelOptions,
    ResponseModelStatus,
    ResponseModelUploadUrl,
)

__all__ = [
    ModelBase,
    InputLayer,
    ModelStatus,
    RequestModelUploadUrl,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelUploadUrl,
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelStatus,
    ResponseModelOptions,
]



from enum import Enum


class EnvironmentType(str, Enum):
    V2_PROD_CLOUD = "v2-prod-cloud"
    V2_PROD_ON_PREM = "v2-prod-on-prem"
    V2_STAGING_CLOUD = "v2-staging-cloud"
    V2_STAGING_ON_PREM = "v2-staging-on-prem"
    V2_DEV_CLOUD = "v2-dev-cloud"
    V2_DEV_ON_PREM = "v2-dev-on-prem"


class ServiceName(str, Enum):
    NP = "NP"
    TAO = "TAO"


class ServiceModule(str, Enum):
    AUTH = "AUTH"
    COMPRESSOR = "COMPRESSOR"
    LAUNCHER = "LAUNCHER"
    TRAINER = "TRAINER"


class EndPointProperty(str, Enum):
    HOST = "HOST"
    PORT = "PORT"
    URI_PREFIX = "URI_PREFIX"



from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Dict, List, Literal, Optional, Union

from omegaconf import MISSING, MissingMandatoryValue

__all__ = [
    "DatasetConfig",
    "LocalClassificationDatasetConfig",
    "LocalSegmentationDatasetConfig",
    "LocalDetectionDatasetConfig",
    "HuggingFaceClassificationDatasetConfig",
    "HuggingFaceSegmentationDatasetConfig",
    "ExampleBeansDataset",
    "ExampleChessDataset",
    "ExampleCocoyoloDataset",
    "ExampleSidewalkDataset",
    "ExampleXrayDataset",
    "ExampleSidewalkDataset",
    "ExampleSkincancerDataset",
    "ExampleTrafficsignDataset",
    "ExampleVoc12Dataset",
    "ExampleVoc12CustomDataset",
    "ExampleWikiartDataset",
]


@dataclass
class DatasetConfig:
    name: str = MISSING
    task: str = MISSING
    format: str = MISSING  # Literal['huggingface', 'local']


@dataclass
class ImageLabelPathConfig:
    image: Optional[Union[Path, str]] = None
    label: Optional[Union[Path, str]] = None


@dataclass
class PathPatternConfig:
    image: Optional[str] = None
    label: Optional[str] = None


@dataclass
class PathConfig:
    root: Union[Path, str] = MISSING
    train: ImageLabelPathConfig = field(default_factory=lambda: ImageLabelPathConfig(image=MISSING))
    valid: ImageLabelPathConfig = field(default_factory=lambda: ImageLabelPathConfig())
    test: ImageLabelPathConfig = field(default_factory=lambda: ImageLabelPathConfig())
    pattern: PathPatternConfig = field(default_factory=lambda: PathPatternConfig())


@dataclass
class HuggingFaceConfig:
    custom_cache_dir: Optional[Union[Path, str]] = None # If None, it follows HF datasets default (.cache/huggingface/datasets)
    repo: str = MISSING
    subset: Optional[str] = None
    features: Dict[str, str] = field(default_factory=lambda: {
        "image": "image", "label": "labels"
    })


@dataclass
class LocalClassificationDatasetConfig(DatasetConfig):
    task: str = "classification"
    format: str = "local"
    path: PathConfig = field(default_factory=lambda: PathConfig())
    id_mapping: Optional[List[str]] = None


@dataclass
class LocalSegmentationDatasetConfig(DatasetConfig):
    task: str = "segmentation"
    format: str = "local"
    path: PathConfig = field(default_factory=lambda: PathConfig())
    label_image_mode: str = "L"
    id_mapping: Any = None
    pallete: Optional[List[List[int]]] = None


@dataclass
class LocalDetectionDatasetConfig(DatasetConfig):
    task: str = "detection"
    format: str = "local"
    path: PathConfig = field(default_factory=lambda: PathConfig())
    id_mapping: Any = None
    pallete: Optional[List[List[int]]] = None


@dataclass
class HuggingFaceClassificationDatasetConfig(DatasetConfig):
    task: str = "classification"
    format: str = "huggingface"
    metadata: HuggingFaceConfig = field(default_factory=lambda: HuggingFaceConfig(
        features={"image": "image", "label": "labels"}
    ))
    id_mapping: Optional[List[str]] = None


@dataclass
class HuggingFaceSegmentationDatasetConfig(DatasetConfig):
    task: str = "segmentation"
    format: str = "huggingface"
    metadata: HuggingFaceConfig = field(default_factory=lambda: HuggingFaceConfig(
        features={"image": "pixel_values", "label": "label"}
    ))
    label_image_mode: str = "L"
    id_mapping: Any = None
    pallete: Optional[List[List[int]]] = None


ExampleBeansDataset = HuggingFaceClassificationDatasetConfig(
    name="beans",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None,
        repo="beans",
        features={"image": "image", "label": "labels"}
    )
)

ExampleChessDataset = LocalClassificationDatasetConfig(
    name="chess",
    path=PathConfig(
        root="/DATA/classification-example",
        train=ImageLabelPathConfig(image="train"),
        valid=ImageLabelPathConfig(image="val"),
    )
)

ExampleXrayDataset = HuggingFaceClassificationDatasetConfig(
    name="chest_xray_classification",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None,
        repo="keremberke/chest-xray-classification",
        subset="full",
        features={"image": "image", "label": "labels"}
    )
)

ExampleCocoyoloDataset = LocalDetectionDatasetConfig(
    name="coco_for_yolo_model",
    path=PathConfig(
        root="/DATA/coco",
        train=ImageLabelPathConfig(image="images/train2017", label="labels/train2017"),
        valid=ImageLabelPathConfig(image="images/train2017", label="labels/train2017"),
        pattern=PathPatternConfig(image="([0-9]{12})\\.jpg", label="([0-9]{12})\\.txt"),
    ),
    id_mapping=[
        'person', 'bicycle', 'car', 'motorcycle', 'airplane', 'bus', 'train', 'truck', 'boat', 'traffic light',
        'fire hydrant', 'stop sign', 'parking meter', 'bench', 'bird', 'cat', 'dog', 'horse', 'sheep', 'cow',
        'elephant', 'bear', 'zebra', 'giraffe', 'backpack', 'umbrella', 'handbag', 'tie', 'suitcase', 'frisbee',
        'skis', 'snowboard', 'sports ball', 'kite', 'baseball bat', 'baseball glove', 'skateboard', 'surfboard',
        'tennis racket', 'bottle', 'wine glass', 'cup', 'fork', 'knife', 'spoon', 'bowl', 'banana', 'apple',
        'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog', 'pizza', 'donut', 'cake', 'chair', 'couch',
        'potted plant', 'bed', 'dining table', 'toilet', 'tv', 'laptop', 'mouse', 'remote', 'keyboard', 'cell phone',
        'microwave', 'oven', 'toaster', 'sink', 'refrigerator', 'book', 'clock', 'vase', 'scissors', 'teddy bear',
        'hair drier', 'toothbrush'
    ]
)

ExampleSidewalkDataset = HuggingFaceSegmentationDatasetConfig(
    name="sidewalk_semantic",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None,
        repo="segments/sidewalk-semantic",
        features={"image": "pixel_values", "label": "label"}
    ),
    label_image_mode="L",
    id_mapping=[
        'unlabeled', 'flat-road', 'flat-sidewalk', 'flat-crosswalk', 'flat-cyclinglane', 'flat-parkingdriveway',
        'flat-railtrack', 'flat-curb', 'human-person', 'human-rider', 'vehicle-car', 'vehicle-truck', 'vehicle-bus',
        'vehicle-tramtrain', 'vehicle-motorcycle', 'vehicle-bicycle', 'vehicle-caravan', 'vehicle-cartrailer',
        'construction-building', 'construction-door', 'construction-wall', 'construction-fenceguardrail',
        'construction-bridge', 'construction-tunnel', 'construction-stairs', 'object-pole', 'object-trafficsign',
        'object-trafficlight', 'nature-vegetation', 'nature-terrain', 'sky', 'void-ground', 'void-dynamic',
        'void-static', 'void-unclear'
    ]
)

ExampleSkincancerDataset = HuggingFaceClassificationDatasetConfig(
    name="skin_cancer",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None,
        repo="marmal88/skin_cancer",
        features={"image": "image", "label": "dx"}
    )
)

ExampleTrafficsignDataset = LocalDetectionDatasetConfig(
    name="traffic_sign_yolo",
    path=PathConfig(
        root="../../data/traffic-sign",
        train=ImageLabelPathConfig(image="images/train", label="labels/train"),
        valid=ImageLabelPathConfig(image="images/val", label="labels/val"),
    ),
    id_mapping=['prohibitory', 'danger', 'mandatory', 'other']  # class names
)

ExampleVoc12Dataset = LocalSegmentationDatasetConfig(
    name="voc2012",
    path=PathConfig(
        root="/DATA/VOC12Dataset",
        train=ImageLabelPathConfig(image="image/train", label="mask/train"),
        valid=ImageLabelPathConfig(image="image/val", label="mask/val"),
    ),
    label_image_mode="RGB",
    id_mapping={
        "(0, 0, 0)": "background",
        "(128, 0, 0)": "aeroplane",
        "(0, 128, 0)": "bicycle",
        "(128, 128, 0)": "bird",
        "(0, 0, 128)": "boat",
        "(128, 0, 128)": "bottle",
        "(0, 128, 128)": "bus",
        "(128, 128, 128)": "car",
        "(64, 0, 0)": "cat",
        "(192, 0, 0)": "chair",
        "(64, 128, 0)": "cow",
        "(192, 128, 0)": "diningtable",
        "(64, 0, 128)": "dog",
        "(192, 0, 128)": "horse",
        "(64, 128, 128)": "motorbike",
        "(192, 128, 128)": "person",
        "(0, 64, 0)": "pottedplant",
        "(128, 64, 0)": "sheep",
        "(0, 192, 0)": "sofa",
        "(128, 192, 0)": "train",
        "(0, 64, 128)": "tvmonitor",
        "(128, 64, 128)": "void"
    }
)

ExampleVoc12CustomDataset = LocalSegmentationDatasetConfig(
    name="voc2012",
    path=PathConfig(
        root="../../data/VOC12Dataset",
        train=ImageLabelPathConfig(image="image/train", label="mask/train"),
        valid=ImageLabelPathConfig(image="image/val", label="mask/val"),
    ),
    label_image_mode="L",
    id_mapping=[
        'background', 'aeroplane', 'bicycle', 'bird', 'boat', 'bottle', 'bus', 'car', 'cat', 'chair',
        'cow', 'diningtable', 'dog', 'horse', 'motorbike', 'person', 'pottedplant', 'sheep', 'sofa',
        'train', 'tvmonitor'
    ],
    pallete=[
        [0, 0, 0], [128, 0, 0], [0, 128, 0], [128, 128, 0], [0, 0, 128], [128, 0, 128], [0, 128, 128],
        [128, 128, 128], [64, 0, 0], [192, 0, 0], [64, 128, 0], [192, 128, 0], [64, 0, 128],
        [192, 0, 128], [64, 128, 128], [192, 128, 128], [0, 64, 0], [128, 64, 0], [0, 192, 0],
        [128, 192, 0], [0, 64, 128]
    ]
)

ExampleWikiartDataset = HuggingFaceClassificationDatasetConfig(
    name="wikiart_artist",
    metadata=HuggingFaceConfig(
        custom_cache_dir=None,
        repo="huggan/wikiart",
        subset="full",
        features={"image": "image", "label": "artist"}
    )
)



import cv2


class ClassificationVisualizer:
    def __init__(self, class_map, pallete=None) -> None:
        self.n = len(class_map)
        self.class_map = class_map

    def draw(self, image, pred, model_input_shape=None, text_scale=0.7, text_thickness=2):
        visualize_image = image.copy()

        class_name = self.class_map[int(pred[0])] # Class is determined with top1 score

        color = (0, 0, 255)
        x1, y1 = 0, 0
        text_size, _ = cv2.getTextSize(str(class_name), cv2.FONT_HERSHEY_SIMPLEX, text_scale, text_thickness)
        text_w, text_h = text_size
        visualize_image = cv2.rectangle(visualize_image, (x1, y1), (x1+text_w, y1+text_h+5), color=color, thickness=-1)
        visualize_image = cv2.putText(visualize_image, str(class_name), (x1, y1+text_h), cv2.FONT_HERSHEY_SIMPLEX, text_scale, (255, 255, 255), text_thickness)

        return visualize_image



from dataclasses import dataclass, field
from typing import List

from netspresso.enums.compression import GroupPolicy, LayerNorm, Policy, StepOp
from netspresso.exceptions.compressor import NotValidChannelAxisRangeException


@dataclass
class OptionsBase:
    reshape_channel_axis: int = -1

    def __post_init__(self):
        valid_values = [0, 1, -1, -2]
        if self.reshape_channel_axis not in valid_values:
            raise NotValidChannelAxisRangeException(self.reshape_channel_axis)


@dataclass
class Options(OptionsBase):
    policy: Policy = Policy.AVERAGE
    layer_norm: LayerNorm = LayerNorm.STANDARD_SCORE
    group_policy: GroupPolicy = GroupPolicy.AVERAGE
    step_size: int = 2
    step_op: StepOp = StepOp.ROUND
    reverse: bool = False


@dataclass
class RecommendationOptions(Options):
    min_num_of_value: int = 8


@dataclass
class Layer:
    use: bool = False
    name: str = "input"
    channels: List[int] = field(default_factory=list)
    values: List[int] = field(default_factory=list)



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.enums.metadata import TaskType
from netspresso.enums.model import DataType, Framework
from netspresso.metadata.common import AvailableOption, BaseMetadata, InputShape
from netspresso.metadata.trainer import TrainingInfo


@dataclass
class ModelInfo:
    task: str = ""
    model: str = ""
    dataset: str = ""
    data_type: DataType = DataType.FP32
    framework: Framework = ""
    input_shapes: List[InputShape] = field(default_factory=list)


@dataclass
class CompressionInfo:
    method: str = ""
    ratio: float = 0.0
    options: Dict[str, Any] = None
    layers: List[Dict] = field(default_factory=list)


@dataclass
class Model:
    size: int = 0
    flops: int = 0
    number_of_parameters: int = 0
    trainable_parameters: int = 0
    non_trainable_parameters: int = 0
    number_of_layers: Optional[int] = None
    model_id: str = ""


@dataclass
class Results:
    original_model: Model = field(default_factory=Model)
    compressed_model: Model = field(default_factory=Model)


@dataclass
class CompressorMetadata(BaseMetadata):
    task_type: TaskType = TaskType.COMPRESS
    input_model_path: str = ""
    compressed_model_path: str = ""
    compressed_onnx_model_path: str = ""
    is_retrainable: bool = False
    model_info: ModelInfo = field(default_factory=ModelInfo)
    training_info: TrainingInfo = field(default_factory=TrainingInfo)
    compression_info: CompressionInfo = field(default_factory=CompressionInfo)
    results: Results = field(default_factory=Results)
    available_options: List[AvailableOption] = field(default_factory=list)

    def update_is_retrainable(self, is_retrainable):
        self.is_retrainable = is_retrainable

    def update_model_info(self, framework, input_shapes):
        self.model_info.framework = framework
        self.model_info.input_shapes = input_shapes

    def update_model_info_for_trainer(self, task, model, dataset):
        self.model_info.task = task
        self.model_info.model = model
        self.model_info.dataset = dataset

    def update_training_info(self, epochs, batch_size, learning_rate, optimizer):
        self.training_info.epochs = epochs
        self.training_info.batch_size = batch_size
        self.training_info.learning_rate = learning_rate
        self.training_info.optimizer = optimizer

    def update_compression_info(self, method, options, layers, ratio=0.0):
        self.compression_info.method = method
        self.compression_info.ratio = ratio
        self.compression_info.options = options
        self.compression_info.layers = layers

    def update_input_model_path(self, input_model_path):
        self.input_model_path = input_model_path

    def update_compressed_model_path(self, compressed_model_path):
        self.compressed_model_path = compressed_model_path

    def update_compressed_onnx_model_path(self, compressed_onnx_model_path):
        self.compressed_onnx_model_path = compressed_onnx_model_path

    def update_results(self, model, compressed_model):
        def update_model_fields(target, source):
            target.size = source.file_size_in_mb
            target.flops = source.detail.flops
            target.number_of_parameters = source.detail.trainable_parameters + source.detail.non_trainable_parameters
            target.trainable_parameters = source.detail.trainable_parameters
            target.non_trainable_parameters = source.detail.non_trainable_parameters
            target.number_of_layers = source.detail.number_of_layers if source.detail.number_of_layers != 0 else None
            target.model_id = source.ai_model_id

        update_model_fields(self.results.original_model, model)
        update_model_fields(self.results.compressed_model, compressed_model)

    def update_available_options(self, available_options):
        self.available_options = [available_option.to() for available_option in available_options]



from dataclasses import dataclass, field
from typing import List

from netspresso.clients.compressor.v2.schemas.compression.base import Layer, Options, RecommendationOptions
from netspresso.enums.compression import CompressionMethod, RecommendationMethod
from netspresso.exceptions.compressor import (
    EmptyCompressionParamsException,
    NotValidSlampRatioException,
    NotValidVbmfRatioException,
)


@dataclass
class RequestCreateCompression:
    ai_model_id: str
    compression_method: CompressionMethod
    options: Options = field(default_factory=Options)


@dataclass
class RequestCreateRecommendation:
    recommendation_method: RecommendationMethod
    recommendation_ratio: float
    options: RecommendationOptions = field(default_factory=RecommendationOptions)

    def __post_init__(self):
        if self.recommendation_method in [RecommendationMethod.SLAMP] and not 0 < self.recommendation_ratio < 1:
            raise NotValidSlampRatioException(ratio=self.recommendation_ratio)
        elif self.recommendation_method in [RecommendationMethod.VBMF] and not -1 <= self.recommendation_ratio <= 1:
            raise NotValidVbmfRatioException(ratio=self.recommendation_ratio)


@dataclass
class RequestUpdateCompression:
    available_layers: List[Layer]
    options: Options = field(default_factory=Options)

    def __post_init__(self):
        if all(not available_layer.values for available_layer in self.available_layers):
            raise EmptyCompressionParamsException()


@dataclass
class RequestAutomaticCompressionParams:
    compression_ratio: float = 0.5


@dataclass
class RequestAvailableLayers:
    compression_method: CompressionMethod
    options: Options = field(default_factory=Options)



import dataclasses
from dataclasses import dataclass, field
from typing import Dict, List, Optional

from netspresso.clients.compressor.v2.schemas.common import ResponseItem, ResponsePaginationItems
from netspresso.clients.compressor.v2.schemas.compression.base import Layer, Options
from netspresso.enums.compression import CompressionMethod, RecommendationMethod


@dataclass
class ResponseCompression:
    compression_id: str
    compression_method: CompressionMethod
    is_completed: bool
    is_deleted: bool
    input_model_id: str
    original_model_id: str
    user_id: str
    available_layers: List[Layer]
    options: Dict

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.available_layers = [Layer(**layer) for layer in self.available_layers]


@dataclass
class ResponseRecommendation:
    recommendation_id: str
    recommendation_method: RecommendationMethod
    recommendation_ratio: float
    available_layers: List[Layer]
    options: Dict
    compression_id: str

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.available_layers = [Layer(**layer) for layer in self.available_layers]


@dataclass
class ResponseSelectMethod:
    input_model_id: str
    compression_method: CompressionMethod
    options: Options
    available_layers: List[Layer]

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.available_layers = [Layer(**layer) for layer in self.available_layers]


@dataclass
class ResponseCompressionItem(ResponseItem):
    data: Optional[ResponseCompression] = field(default_factory=dict)

    def __post_init__(self):
        self.data = ResponseCompression(**self.data)


@dataclass
class ResponseCompressionItems(ResponsePaginationItems):
    data: List[ResponseCompression] = field(default_factory=list)

    def __post_init__(self):
        self.data = [ResponseCompression(**compression) for compression in self.data]


@dataclass
class ResponseRecommendationItem(ResponseItem):
    data: Optional[ResponseRecommendation] = field(default_factory=dict)

    def __post_init__(self):
        self.data = ResponseRecommendation(**self.data)


@dataclass
class ResponseSelectMethodItem(ResponseItem):
    data: Optional[ResponseSelectMethod] = None

    def __post_init__(self):
        self.data = ResponseSelectMethod(**self.data)



from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import List, Optional, Union

from netspresso.clients.utils.system import ENV_STR
from netspresso.enums import (
    DataType,
    DisplaySoftwareVersion,
    Framework,
    HardwareType,
    SoftwareVersion,
)
from netspresso.metadata import common
from netspresso.metadata.common import AvailableOption, SoftwareVersions

version = (Path(__file__).parent.parent.parent.parent.parent / "VERSION").read_text().strip()


class Order(str, Enum):
    """ """

    DESC = "desc"
    ASC = "asc"


@dataclass
class AuthorizationHeader:
    Authorization: str

    def __init__(self, access_token):
        self.Authorization = f"Bearer {access_token}"

    def to_dict(self):
        return {
            "Authorization": self.Authorization,
            "User-Agent": f"NetsPresso Python Package v{version} ({ENV_STR})"
        }

@dataclass
class UploadFile:
    files: List

    def __init__(self, file_name, file_content):
        self.files = [("file", (file_name, file_content))]


@dataclass
class UploadDataset:
    files: List

    def __init__(self, file_name, file_content):
        self.files = [("dataset", (file_name, file_content))]


@dataclass
class RequestPagination:
    """ """

    start: int = 0
    size: int = 10
    order: Order = Order.DESC.value
    paging: bool = True


@dataclass
class ResponseItem:
    """ """

    data: Optional[object] = field(default_factory=dict)


@dataclass
class ResponseItems:
    """ """

    data: List[Optional[object]] = field(default_factory=list)


@dataclass
class ResponsePaginationItems:
    """ """

    result_count: int
    total_count: int
    data: List[Optional[object]] = field(default_factory=list)


@dataclass
class SoftwareVersionInfo:
    """ """

    software_version: Optional[Union[None, SoftwareVersion]] = None
    display_software_version: Optional[Union[None, DisplaySoftwareVersion]] = None

    def to(self) -> SoftwareVersions:
        software_version = SoftwareVersions()

        software_version.software_version = self.software_version
        software_version.display_software_versions = self.display_software_version

        return software_version


@dataclass
class TaskInfo:
    """ """
    device_name: str
    display_brand_name: str
    display_device_name: str
    software_version: Optional[SoftwareVersion]
    display_software_version: Optional[DisplaySoftwareVersion]
    data_type: DataType
    hardware_type: Optional[HardwareType]


@dataclass
class DeviceInfo:
    """ """

    device_name: str
    display_device_name: str
    display_brand_name: str
    software_versions: Optional[List[SoftwareVersionInfo]] = field(default_factory=list)
    data_types: Optional[List[DataType]] = field(default_factory=list)
    hardware_types: Optional[List[HardwareType]] = field(default_factory=list)

    def __post_init__(self):
        self.software_versions = [SoftwareVersionInfo(**item) for item in self.software_versions]

    def to(self) -> common.DeviceInfo:
        device_info = common.DeviceInfo()

        device_info.device_name = self.device_name
        device_info.display_device_name = self.display_device_name
        device_info.display_brand_name = self.display_brand_name
        device_info.data_types = self.data_types
        device_info.hardware_types = self.hardware_types

        device_info.software_versions = [item.to() for item in self.software_versions]

        return device_info

@dataclass
class TaskOption:
    """ """

    framework: Optional[Framework] = ""
    display_framework: Optional[str] = ""
    target_device: Optional[TaskInfo] = field(default_factory=dict)

    def __post_init__(self):
        self.target_device = TaskInfo(**self.target_device)


@dataclass
class ModelOption:
    """ """

    framework: Optional[Framework] = ""
    display_framework: Optional[str] = ""
    devices: List[DeviceInfo] = field(default_factory=list)

    def __post_init__(self):
        self.devices = [DeviceInfo(**item) for item in self.devices]

    def to(self) -> AvailableOption:
        available_options = AvailableOption()
        available_options.framework = self.framework
        available_options.display_framework = self.display_framework
        for device in self.devices:
            available_options.devices.append(device.to())

        return available_options



import matplotlib.pyplot as plt
import numpy as np

from netspresso.metadata.compressor import CompressorMetadata
from netspresso.metadata.trainer import TrainerMetadata


class Plotter:
    BAR_WIDTH = 0.45

    @staticmethod
    def _add_text_annotation(ax, x_point, y_point, text):
        ax.annotate(
            f"{text:.2f}",
            (x_point, y_point),
            textcoords="offset points",
            xytext=(0, 5),
            ha="center",
            va="bottom",
        )

    @staticmethod
    def _add_value_annotations(ax, bar, value):
        ax.text(
            bar.get_x() + bar.get_width() / 2,
            bar.get_height() + 0.01,
            f"{value:.4f}",
            ha="center",
            va="bottom",
        )

    @staticmethod
    def _add_difference_annotations(ax, original_value, compressed_value, difference):
        ax.scatter(
            ["Original Model", "Compressed Model"],
            [original_value, compressed_value],
            color="red",
            marker="o",
            zorder=3,
        )

        ax.plot(
            ["Original Model", "Compressed Model"],
            [original_value, compressed_value],
            color="red",
            linestyle="--",
            linewidth=2,
            zorder=2,
        )

        diff_x = np.mean(ax.get_xlim())
        ax.text(
            diff_x,
            original_value * 0.5,
            f"Difference: {difference:.4f}",
            ha="center",
            va="bottom",
            color="black",
        )

    @staticmethod
    def _plot_single_bar(ax, label, value, color):
        return ax.bar(
            [label],
            [value],
            color=color,
            label=label,
            width=Plotter.BAR_WIDTH,
        )

    @staticmethod
    def _set_common_plot_settings(ax, ylabel, ylim_max=1):
        ax.set_ylim(0, ylim_max)
        ax.set_ylabel(ylabel)
        ax.legend()
        ax.grid(axis="y")

    @staticmethod
    def _plot_comparison(original_value, value_per_model, target_value, title, xlabel, ylabel):
        ratios = list(value_per_model.keys())
        values = list(value_per_model.values())
        plt.figure(figsize=(15, 6))
        plt.plot(ratios, values, marker="o", label="Compressed Model")
        plt.axhline(original_value, color="slategray", linestyle="--", label="Original Model")
        if target_value:
            plt.axhline(target_value, color="red", linestyle="--", label="Target Value")
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.legend()
        plt.grid(True)

        for ratio, value in value_per_model.items():
            Plotter._add_text_annotation(plt.gca(), ratio, value, value)

    @staticmethod
    def compare_latency(original_latency, latency_per_model, target_latency):
        Plotter._plot_comparison(
            original_latency,
            latency_per_model,
            target_latency,
            "Latency vs. Compression Ratio",
            "Compression Ratio",
            "Latency (ms)",
        )

    @staticmethod
    def compare_flops(original_flops, flops_per_model):
        original_flops = round(original_flops/1e9, 2)
        for key, value in flops_per_model.items():
            flops_per_model[key] = round(value/1e9, 2)
        Plotter._plot_comparison(
            original_flops,
            flops_per_model,
            None,
            "FLOPs vs. Compression Ratio",
            "Compression Ratio",
            "FLOPs (G)",
        )

    @staticmethod
    def compare_metric(original_summary: TrainerMetadata, compressed_summary: TrainerMetadata):
        original_training_result = original_summary.training_result
        compressed_training_result = compressed_summary.training_result
        metrics_list = original_training_result["metrics_list"]
        original_best_epoch = str(original_training_result["best_epoch"])
        compressed_best_epoch = str(compressed_training_result["best_epoch"])
        original_best_metrics = list(original_training_result["valid_metrics"][original_best_epoch].values())
        compressed_best_metrics = list(compressed_training_result["valid_metrics"][compressed_best_epoch].values())

        fig, axs = plt.subplots(ncols=len(metrics_list), figsize=(15, 6))

        for idx, _metric in enumerate(metrics_list):
            labels = {
                "map50": "mAP@[.50]",
                "map75": "mAP@[.75]",
                "map50_95": "mAP@[.50:.95]",
            }
            metric = labels[_metric]
            bars_original = Plotter._plot_single_bar(
                axs[idx], "Original Model", original_best_metrics[idx], "slategray"
            )
            bars_compressed = Plotter._plot_single_bar(
                axs[idx], "Compressed Model", compressed_best_metrics[idx], "dodgerblue"
            )

            for bar in bars_original:
                Plotter._add_value_annotations(axs[idx], bar, original_best_metrics[idx])

            for bar in bars_compressed:
                Plotter._add_value_annotations(axs[idx], bar, compressed_best_metrics[idx])

            Plotter._add_difference_annotations(
                axs[idx],
                original_best_metrics[idx],
                compressed_best_metrics[idx],
                compressed_best_metrics[idx] - original_best_metrics[idx],
            )

            Plotter._set_common_plot_settings(axs[idx], metric)

        plt.tight_layout()
        plt.show()

    @staticmethod
    def compare_profile_result(profile_result: CompressorMetadata):
        y_labels = ["FLOPs(G)", "Num of Params(M)", "Model Size(MB)"]

        original_model = profile_result.results.original_model
        compressed_model = profile_result.results.compressed_model

        original_values = [
            round(original_model.flops/1e9, 2),
            round(original_model.number_of_parameters/1e6, 2),
            original_model.size
        ]
        compressed_values = [
            round(compressed_model.flops/1e9, 2),
            round(compressed_model.number_of_parameters/1e6, 2),
            compressed_model.size
        ]

        difference_values = np.array(original_values) / np.array(compressed_values)

        fig, axs = plt.subplots(ncols=len(y_labels), figsize=(15, 6))

        for idx, label in enumerate(y_labels):
            bars_original = axs[idx].bar(
                ["Original Model"],
                [original_values[idx]],
                color="slategray",
                label="Original Model",
                width=Plotter.BAR_WIDTH,
            )
            bars_compressed = axs[idx].bar(
                ["Compressed Model"],
                [compressed_values[idx]],
                color="dodgerblue",
                label="Compressed Model",
                width=Plotter.BAR_WIDTH,
            )

            for bar in bars_original:
                axs[idx].text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() + 0.01,
                    f"{bar.get_height():.2f}",
                    ha="center",
                    va="bottom",
                )

            for bar in bars_compressed:
                axs[idx].text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() + 0.01,
                    f"{bar.get_height():.2f}",
                    ha="center",
                    va="bottom",
                )

            axs[idx].scatter(
                ["Original Model", "Compressed Model"],
                [original_values[idx], compressed_values[idx]],
                color="red",
                marker="o",
                zorder=3,
            )

            axs[idx].plot(
                ["Original Model", "Compressed Model"],
                [original_values[idx], compressed_values[idx]],
                color="red",
                linestyle="--",
                linewidth=2,
                zorder=2,
            )

            diff_x = np.mean(axs[idx].get_xlim())
            axs[idx].text(
                diff_x,
                original_values[idx] * 0.5,
                f"{difference_values[idx]:.4f}x",
                ha="center",
                va="bottom",
                color="black",
            )

            axs[idx].set_ylabel(label)
            axs[idx].legend()
            axs[idx].grid(axis="y")

        plt.tight_layout()
        plt.show()

    @staticmethod
    def _plot_epoch_metrics(train_data, valid_data, title, xlabel, ylabel):
        labels = {
            "map50": "mAP@[.50]",
            "map75": "mAP@[.75]",
            "map50_95": "mAP@[.50:.95]",
        }
        metric = labels[ylabel]

        train_epochs = list(train_data.keys())
        valid_epochs = list(valid_data.keys())

        train_metric_values = [metric[ylabel] for _, metric in train_data.items()]
        valid_metric_values = [metric[ylabel] for _, metric in valid_data.items()]

        plt.figure(figsize=(15, 6))
        plt.plot(train_epochs, train_metric_values, label="Train Metric", marker="o", color="slategray")
        plt.plot(valid_epochs, valid_metric_values, label="Validation Metric", marker="x", color="dodgerblue")
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(metric)
        plt.legend()
        plt.grid(True)
        max_value = max(train_metric_values + valid_metric_values)
        plt.ylim(0, max_value * 1.2)

        for epoch, metric in valid_data.items():
            metric = metric[ylabel]
            Plotter._add_text_annotation(plt.gca(), epoch, metric, metric)

        plt.show()

    @staticmethod
    def plot_metric_by_epoch(train_data, valid_data, title="Train and Validation metric per epoch", xlabel="Epochs"):
        for _ylabel in ["map50", "map75", "map50_95"]:
            Plotter._plot_epoch_metrics(train_data, valid_data, title, xlabel, _ylabel)

    @staticmethod
    def _plot_epoch_losses(train_data, valid_data, title, xlabel, ylabel):
        train_epochs = list(train_data.keys())
        valid_epochs = list(valid_data.keys())

        train_loss_values = list(train_data.values())
        valid_loss_values = list(valid_data.values())

        plt.figure(figsize=(15, 6))
        plt.plot(train_epochs, train_loss_values, label="Train Loss", marker="o", color="slategray")
        plt.plot(valid_epochs, valid_loss_values, label="Validation Loss", marker="x", color="dodgerblue")
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.legend()
        plt.grid(True)
        max_value = max(train_loss_values + valid_loss_values)
        plt.ylim(0, max_value * 1.2)

        for epoch, loss in valid_data.items():
            Plotter._add_text_annotation(plt.gca(), epoch, loss, loss)

        plt.show()

    @staticmethod
    def plot_loss_by_epoch(
        train_data, valid_data, title="Train and Validation loss per epoch", xlabel="Epochs", ylabel="Loss"
    ):
        Plotter._plot_epoch_losses(train_data, valid_data, title, xlabel, ylabel)

    @staticmethod
    def compare_by_step_size(
        a_compression_result, b_compression_result, a_benchmark_result, b_benchmark_result, x_labels=None
    ):
        if x_labels is None:
            x_labels = [2, 32]
        x_labels = [f"step_size={x_label}" for x_label in x_labels]
        y_labels = ["Latency(ms)", "FLOPs(M)", "Num of Params(M)", "Model Size(MB)"]
        keys = ["latency", "flops", "number_of_parameters", "size"]

        a_values = [a_benchmark_result["result"]["latency"]]
        b_values = [b_benchmark_result["result"]["latency"]]

        for _key in keys[1:]:
            a_values.append(a_compression_result["results"]["compressed_model"][_key])

        for _key in keys[1:]:
            b_values.append(b_compression_result["results"]["compressed_model"][_key])

        fig, axs = plt.subplots(ncols=len(y_labels), figsize=(15, 5))

        for idx, label in enumerate(y_labels):
            bars_original = axs[idx].bar(
                [x_labels[0]],
                [a_values[idx]],
                color="slategray",
                label=x_labels[0],
                width=Plotter.BAR_WIDTH,
            )
            bars_compressed = axs[idx].bar(
                [x_labels[1]],
                [b_values[idx]],
                color="dodgerblue",
                label=x_labels[1],
                width=Plotter.BAR_WIDTH,
            )

            for bar in bars_original:
                axs[idx].text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() + 0.01,
                    f"{bar.get_height():.4f}",
                    ha="center",
                    va="bottom",
                )

            for bar in bars_compressed:
                axs[idx].text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() + 0.01,
                    f"{bar.get_height():.4f}",
                    ha="center",
                    va="bottom",
                )

            axs[idx].set_ylabel(label)
            axs[idx].set_title(f"Step Size vs. {label}")
            axs[idx].legend()
            axs[idx].grid(axis="y")
            max_value = max(a_values[idx], b_values[idx])
            axs[idx].set_ylim(max_value * 0.8, max_value * 1.1)

        plt.tight_layout()
        plt.show()

    @staticmethod
    def plot_profile_results(data, x_labels, title_prefix, y_labels=None):
        if y_labels is None:
            y_labels = ["latency", "flops", "params", "size"]
        y_label_names = {
            "latency": "Latency(ms)",
            "flops": "FLOPs(M)",
            "params": "Num of Params(M)",
            "size": "Model Size(MB)",
        }

        fig, axs = plt.subplots(nrows=1, ncols=4, figsize=(15, 5))

        for idx, label in enumerate(y_labels):
            y_label_name = y_label_names[label]
            axs[idx].plot(x_labels, data[label], marker="o", color="dodgerblue", label=label)

            axs[idx].set_xlabel("Step Size" if "Step" in title_prefix else "Compression Ratio")
            axs[idx].set_ylabel(y_label_name)
            axs[idx].set_title(f"{title_prefix} vs. {y_label_name}")
            axs[idx].grid(True)

            max_value = max(data[label])
            axs[idx].set_ylim(0, max_value * 1.2)
            axs[idx].set_xlim(-0.5, len(x_labels) - 0.5)

            for i, value in enumerate(data[label]):
                axs[idx].text(i, value, f"{value:.2f}", ha="center", va="bottom")

        plt.tight_layout()
        plt.show()

    @staticmethod
    def prepare_data(original_benchmark_result, compressed_benchmark_result, profile_result):
        # Original and compressed latency values
        original_latency = original_benchmark_result["result"]["latency"]
        compressed_latency = compressed_benchmark_result["result"]["latency"]
        original_values = [original_latency]
        compressed_values = [compressed_latency]

        # FLOPs, number of parameters, and model size
        keys = ["flops", "number_of_parameters", "size"]
        original_values.extend(profile_result["results"]["original_model"][_key] for _key in keys)
        compressed_values.extend(profile_result["results"]["compressed_model"][_key] for _key in keys)

        return original_values, compressed_values

    @staticmethod
    def summary_profile_results(original_benchmark_result, compressed_benchmark_result, profile_result):
        original_values, compressed_values = Plotter.prepare_data(
            original_benchmark_result, compressed_benchmark_result, profile_result
        )
        y_labels = ["Latency(ms)", "FLOPs(M)", "Num of Params(M)", "Model Size(MB)"]

        difference_values = np.array(original_values) / np.array(compressed_values)

        fig, axs = plt.subplots(ncols=len(y_labels), figsize=(15, 6))

        for idx, label in enumerate(y_labels):
            bars_original = axs[idx].bar(
                ["Original Model"],
                [original_values[idx]],
                color="slategray",
                label="Original Model",
                width=Plotter.BAR_WIDTH,
            )
            bars_compressed = axs[idx].bar(
                ["Compressed Model"],
                [compressed_values[idx]],
                color="dodgerblue",
                label="Compressed Model",
                width=Plotter.BAR_WIDTH,
            )

            for bar in bars_original:
                axs[idx].text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() + 0.01,
                    f"{bar.get_height():.4f}",
                    ha="center",
                    va="bottom",
                )

            for bar in bars_compressed:
                axs[idx].text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height() + 0.01,
                    f"{bar.get_height():.4f}",
                    ha="center",
                    va="bottom",
                )

            axs[idx].scatter(
                ["Original Model", "Compressed Model"],
                [original_values[idx], compressed_values[idx]],
                color="red",
                marker="o",
                zorder=3,
            )

            axs[idx].plot(
                ["Original Model", "Compressed Model"],
                [original_values[idx], compressed_values[idx]],
                color="red",
                linestyle="--",
                linewidth=2,
                zorder=2,
            )

            diff_x = np.mean(axs[idx].get_xlim())
            axs[idx].text(
                diff_x,
                original_values[idx] * 0.5,
                f"{difference_values[idx]:.4f}x",
                ha="center",
                va="bottom",
                color="black",
            )

            axs[idx].set_ylabel(label)
            axs[idx].legend()
            axs[idx].grid(axis="y")
            max_value = max([original_values[idx], compressed_values[idx]])
            axs[idx].set_ylim(0, max_value * 1.2)

        plt.tight_layout()
        plt.show()

    @staticmethod
    def line_plot_overall_latencies(step_sizes, compression_ratios, latencies, original_latency, target_latency):
        plt.figure(figsize=(10, 6))

        for i, step_size in enumerate(step_sizes):
            plt.plot(compression_ratios, latencies[i], marker="o", label=f"Step Size={step_size}")
            for j, latency in enumerate(latencies[i]):
                plt.text(compression_ratios[j], latency, f"{latency:.2f}", ha="center", va="bottom")

        plt.xlabel("Compression Ratio")
        plt.ylabel("Latency (ms)")
        plt.title("Latency vs. Compression Ratio for Different Step Sizes")
        plt.axhline(
            original_latency, color="slategray", linestyle="--", label=f"Original Latency: {original_latency:.2f} ms"
        )
        if target_latency:
            plt.axhline(target_latency, color="red", linestyle="--", label=f"Target Latency: {target_latency:.2f} ms")
        plt.legend()
        plt.grid(True)
        plt.xticks(compression_ratios)

        plt.tight_layout()
        plt.show()

    @staticmethod
    def bar_plot_overall_latencies(step_sizes, compression_ratios, latencies):
        fig, ax = plt.subplots(figsize=(10, 6))
        x = np.arange(len(compression_ratios))
        width = 0.35

        for i, step_size in enumerate(step_sizes):
            bars = ax.bar(x + i * width, latencies[i], width, label=f"Step Size={step_size}")
            for bar in bars:
                ax.text(
                    bar.get_x() + bar.get_width() / 2,
                    bar.get_height(),
                    f"{bar.get_height():.2f}",
                    ha="center",
                    va="bottom",
                )

        ax.set_xlabel("Compression Ratio")
        ax.set_ylabel("Latency")
        ax.set_title("Latency for Different Compression Ratios and Step Sizes")
        ax.set_xticks(x + width / 2)
        ax.set_xticklabels(compression_ratios)
        ax.legend()
        ax.grid(True)

        plt.tight_layout()
        plt.show()

    @staticmethod
    def bar_plot_overall_latencies_by_device(model_names, latency_of_models, devices):
        for model, data in zip(model_names, latency_of_models):
            fig, ax = plt.subplots(figsize=(12, 6))
            bars = ax.bar(devices, data, color="dodgerblue")
            ax.set_xlabel("Devices")
            ax.set_ylabel("Latency")
            ax.set_title(f"Latency of {model} on Different Devices")
            ax.grid(True)
            plt.xticks(rotation=90)
            max_value = max(data)
            ax.set_ylim(0, max_value * 1.2)

            for bar in bars:
                height = bar.get_height()
                ax.text(bar.get_x() + bar.get_width() / 2, height, f"{height:.2f}", ha="center", va="bottom")

            plt.tight_layout()
            plt.show()



from enum import Enum


class ServiceTask(str, Enum):
    TRAINING = "Training"
    ADVANCED_COMPRESSION = "Advanced Compression"
    AUTOMATIC_COMPRESSION = "Automatic Compression"
    MODEL_CONVERT = "Conversion"
    MODEL_BENCHMARK = "Benchmark"


class ServiceCredit:
    CREDITS = {
        ServiceTask.ADVANCED_COMPRESSION: 50,
        ServiceTask.AUTOMATIC_COMPRESSION: 25,
        ServiceTask.MODEL_CONVERT: 50,
        ServiceTask.MODEL_BENCHMARK: 25,
    }

    @staticmethod
    def get_credit(task_id):
        return ServiceCredit.CREDITS.get(task_id, "Task not found")


class MembershipType(str, Enum):
    BASIC = "BASIC"
    PRO = "PRO"
    PREMIUM = "PREMIUM"



from dataclasses import dataclass, field
from enum import Enum
from typing import List, Optional, Union

from netspresso.enums.device import DisplaySoftwareVersion, HardwareType, SoftwareVersion
from netspresso.enums.model import DataType, Framework
from netspresso.metadata import common
from netspresso.metadata.common import AvailableOption


class Order(str, Enum):
    """ """

    DESC = "desc"
    ASC = "asc"


@dataclass
class AuthorizationHeader:
    Authorization: str

    def __init__(self, access_token):
        self.Authorization = f"Bearer {access_token}"


@dataclass
class UploadFile:
    files: List

    def __init__(self, file_name, file_content):
        self.files = [("file", (file_name, file_content))]


@dataclass
class RequestPagination:
    """ """

    start: int = 0
    size: int = 10
    order: Order = Order.DESC.value
    paging: bool = True


@dataclass
class ResponseItem:
    """ """

    data: Optional[object] = field(default_factory=dict)


@dataclass
class ResponseItems:
    """ """

    data: List[Optional[object]] = field(default_factory=list)


@dataclass
class ResponsePaginationItems:
    """ """

    result_count: int
    total_count: int
    data: List[Optional[object]] = field(default_factory=list)


@dataclass
class SoftwareVersionInfo:
    """ """

    software_version: Optional[Union[None, SoftwareVersion]] = None
    display_software_version: Optional[Union[None, DisplaySoftwareVersion]] = None


@dataclass
class DeviceInfo:
    """ """

    device_name: str
    display_device_name: str
    display_brand_name: str
    software_versions: Optional[List[SoftwareVersionInfo]] = field(default_factory=list)
    data_types: Optional[List[DataType]] = field(default_factory=list)
    hardware_types: Optional[List[HardwareType]] = field(default_factory=list)

    def __post_init__(self):
        self.software_versions = [SoftwareVersionInfo(**item) for item in self.software_versions]

    def to(self) -> common.DeviceInfo:
        device_info = common.DeviceInfo()

        device_info.device_name = self.device_name
        device_info.display_device_name = self.display_device_name
        device_info.display_brand_name = self.display_brand_name
        device_info.data_types = self.data_types
        device_info.hardware_types = self.hardware_types

        device_info.software_versions.software_version = self.software_versions[0].software_version
        device_info.software_versions.display_software_versions = self.software_versions[0].display_software_version

        return device_info


@dataclass
class ModelOption:
    """ """

    framework: Optional[Framework] = ""
    display_framework: Optional[str] = ""
    devices: List[DeviceInfo] = field(default_factory=list)

    def __post_init__(self):
        self.devices = [DeviceInfo(**item) for item in self.devices]

    def to(self) -> AvailableOption:
        available_options = AvailableOption()
        available_options.framework = self.framework
        available_options.display_framework = self.display_framework
        for device in self.devices:
            available_options.devices.append(device.to())

        return available_options



import sys


def check_credit_balance(user_credit, service_credit):
    service_name = service_credit.name.replace("_", " ").lower()
    if user_credit < service_credit:
        sys.exit(
            f"Your current balance of {user_credit} credits is insufficient to complete the task. \n{service_credit} credits are required for one {service_name} task. \nFor additional credit, please contact us at netspresso@nota.ai."
        )



from netspresso.trainer.models.base import CheckpointConfig, ModelConfig
from netspresso.trainer.models.efficientformer import (
    ClassificationEfficientFormerModelConfig,
    DetectionEfficientFormerModelConfig,
    SegmentationEfficientFormerModelConfig,
)
from netspresso.trainer.models.mixnet import (
    ClassificationMixNetLargeModelConfig,
    ClassificationMixNetMediumModelConfig,
    ClassificationMixNetSmallModelConfig,
    DetectionMixNetLargeModelConfig,
    DetectionMixNetMediumModelConfig,
    DetectionMixNetSmallModelConfig,
    SegmentationMixNetLargeModelConfig,
    SegmentationMixNetMediumModelConfig,
    SegmentationMixNetSmallModelConfig,
)
from netspresso.trainer.models.mobilenetv3 import (
    ClassificationMobileNetV3LargeModelConfig,
    ClassificationMobileNetV3SmallModelConfig,
    DetectionMobileNetV3SmallModelConfig,
    SegmentationMobileNetV3SmallModelConfig,
)
from netspresso.trainer.models.mobilevit import ClassificationMobileViTModelConfig
from netspresso.trainer.models.pidnet import PIDNetModelConfig
from netspresso.trainer.models.resnet import (
    ClassificationResNet18ModelConfig,
    ClassificationResNet34ModelConfig,
    ClassificationResNet50ModelConfig,
    DetectionResNet50ModelConfig,
    SegmentationResNet50ModelConfig,
)
from netspresso.trainer.models.rtmpose import PoseEstimationMobileNetV3SmallModelConfig
from netspresso.trainer.models.segformer import SegmentationSegFormerB0ModelConfig
from netspresso.trainer.models.vit import ClassificationViTTinyModelConfig
from netspresso.trainer.models.yolo import DetectionYoloFastestModelConfig
from netspresso.trainer.models.yolox import (
    DetectionYoloXLModelConfig,
    DetectionYoloXMModelConfig,
    DetectionYoloXSModelConfig,
    DetectionYoloXXModelConfig,
)

CLASSIFICATION_MODELS = {
    "EfficientFormer": ClassificationEfficientFormerModelConfig,
    "EfficientFormer-L1": ClassificationEfficientFormerModelConfig,
    "MobileNetV3_Small": ClassificationMobileNetV3SmallModelConfig,
    "MobileNetV3-S": ClassificationMobileNetV3SmallModelConfig,
    "MobileNetV3_Large": ClassificationMobileNetV3LargeModelConfig,
    "MobileNetV3-L": ClassificationMobileNetV3LargeModelConfig,
    "MobileViT": ClassificationMobileViTModelConfig,
    "MobileViT-S": ClassificationMobileViTModelConfig,
    "ResNet18": ClassificationResNet18ModelConfig,
    "ResNet34": ClassificationResNet34ModelConfig,
    "ResNet50": ClassificationResNet50ModelConfig,
    "ViT-Tiny": ClassificationViTTinyModelConfig,
    "ViT-T": ClassificationViTTinyModelConfig,
    "MixNet-S": ClassificationMixNetSmallModelConfig,
    "MixNet-M": ClassificationMixNetMediumModelConfig,
    "MixNet-L": ClassificationMixNetLargeModelConfig,
}

DETECTION_MODELS = {
    "YOLOX-S": DetectionYoloXSModelConfig,
    "YOLOX-M": DetectionYoloXMModelConfig,
    "YOLOX-L": DetectionYoloXLModelConfig,
    "YOLOX-X": DetectionYoloXXModelConfig,
    # "YOLO-Fastest": DetectionYoloFastestModelConfig,
}

SEGMENTATION_MODELS = {
    "EfficientFormer": SegmentationEfficientFormerModelConfig,
    "EfficientFormer-L1": SegmentationEfficientFormerModelConfig,
    "MobileNetV3_Small": SegmentationMobileNetV3SmallModelConfig,
    "MobileNetV3-S": SegmentationMobileNetV3SmallModelConfig,
    "ResNet50": SegmentationResNet50ModelConfig,
    "SegFormer-B0": SegmentationSegFormerB0ModelConfig,
    "MixNet-S": SegmentationMixNetSmallModelConfig,
    "MixNet-M": SegmentationMixNetMediumModelConfig,
    "MixNet-L": SegmentationMixNetLargeModelConfig,
    "PIDNet": PIDNetModelConfig,
    "PIDNet-S": PIDNetModelConfig,
}

POSEESTIMATION_MODELS = {
    "MobileNetV3_Small": PoseEstimationMobileNetV3SmallModelConfig,
    "MobileNetV3-S": PoseEstimationMobileNetV3SmallModelConfig,
}

# NOT_SUPPORTED_PRETRAINED_MODELS = ["YOLO-Fastest"]


__all__ = [
    "CLASSIFICATION_MODELS",
    "DETECTION_MODELS",
    "SEGMENTATION_MODELS",
    "CheckpointConfig",
    "ModelConfig",
]



from dataclasses import asdict

from loguru import logger

from netspresso.clients.auth import response_body
from netspresso.clients.auth.request_body import LoginRequest
from netspresso.clients.auth.v2.schemas.auth import TokenRefreshRequest
from netspresso.clients.auth.v2.schemas.credit import SummarizedCreditResponse
from netspresso.clients.auth.v2.schemas.token import TokenResponse
from netspresso.clients.auth.v2.schemas.user import UserResponse
from netspresso.clients.config import Config
from netspresso.clients.utils.requester import Requester


class AuthClientV2:
    def __init__(self, config: Config):
        """Initialize the UserSession.

        Args:
            email (str): The email address for a user account.
            password (str): The password for a user account.
        """
        self.config = config
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.prefix = self.config.URI_PREFIX
        self.base_url = f"{self.host}:{self.port}{self.prefix}"

    def login(
        self, email, password, verify_ssl: bool = True
    ) -> response_body.TokenResponse:
        try:
            url = f"{self.base_url}/auth/login"
            request_body = LoginRequest(username=email, password=password)
            response = Requester.post_as_form(
                url=url, request_body=asdict(request_body)
            )
            token_response = TokenResponse(**response.json())
            logger.info("Login successfully")
            return token_response.to()
        except Exception as e:
            logger.error(f"Login failed. Error: {e}")
            raise e

    def get_user_info(
        self, access_token, verify_ssl: bool = True
    ) -> response_body.UserResponse:
        user_response = self.__get_user_info(
            access_token=access_token, verify_ssl=verify_ssl
        )
        summarized_credit_response = self.__get_credit(
            access_token=access_token, verify_ssl=verify_ssl
        )
        logger.info("Successfully got user information")
        return user_response.to(summarized_credit_response=summarized_credit_response)

    def __get_user_info(self, access_token, verify_ssl: bool = True) -> UserResponse:
        try:
            url = f"{self.base_url}/users/me"
            headers = self.__make_bearer_header(token=access_token)

            response = Requester.get(url=url, headers=headers)
            return UserResponse(**response.json())
        except Exception as e:
            logger.error(f"Failed to get user information. Error: {e}")
            raise e

    def get_credit(self, access_token, verify_ssl: bool = True) -> int:
        summarized_credit_response = self.__get_credit(
            access_token=access_token, verify_ssl=verify_ssl
        )
        logger.info("Successfully got user credit")
        return summarized_credit_response.data.total_credit

    def __get_credit(
        self, access_token, user_id: str = None, verify_ssl: bool = True
    ) -> SummarizedCreditResponse:
        try:
            if user_id is None:
                user_response = self.__get_user_info(
                    access_token=access_token, verify_ssl=verify_ssl
                )
                user_id = user_response.data.user_id

            url = f"{self.base_url}/users/{user_id}/credit/summarized"
            headers = self.__make_bearer_header(token=access_token)

            response = Requester.get(url=url, headers=headers)
            return SummarizedCreditResponse(**response.json())
        except Exception as e:
            logger.error(f"Failed to get user credit. Error: {e}")
            raise e

    def reissue_token(
        self, access_token, refresh_token, verify_ssl: bool = True
    ) -> response_body.TokenResponse:
        try:
            request_body = TokenRefreshRequest(**{"refresh_token": refresh_token})
            url = f"{self.base_url}/auth/login_by_refresh_token"
            response = Requester.post_as_json(
                url=url, request_body=asdict(request_body)
            )
            logger.info("Successfully reissued token")
            return TokenResponse(**response.json()).to()
        except Exception as e:
            logger.info(f"Failed to reissue token. Error: {e}")
            raise e

    def __make_bearer_header(self, token: str):
        return {"Authorization": f"Bearer {token}"}



from dataclasses import dataclass, field
from typing import List

from netspresso.enums import (
    DataType,
    DeviceName,
    Framework,
    HardwareType,
    SoftwareVersion,
    TaskType,
)
from netspresso.metadata.common import BaseMetadata


@dataclass
class BenchmarkTaskInfo:
    benchmark_task_uuid: str = ""
    framework: Framework = ""
    display_framework: str = ""
    device_name: DeviceName = ""
    display_device_name: str = ""
    display_brand_name: str = ""
    software_version: SoftwareVersion = ""
    display_software_version: str = ""
    data_type: DataType = ""
    hardware_type: HardwareType = ""


@dataclass
class BenchmarkResult:
    memory_footprint: int = None
    memory_footprint_gpu: int = None
    memory_footprint_cpu: int = None
    power_consumption: int = None
    ram_size: int = None
    latency: int = None
    file_size: int = None


@dataclass
class BenchmarkEnvironment:
    model_framework: str = ""
    system: str = ""
    machine: str = ""
    cpu: str = ""
    gpu: str = ""
    library: List[str] = field(default_factory=list)


@dataclass
class BenchmarkerMetadata(BaseMetadata):
    task_type: TaskType = TaskType.BENCHMARK
    input_model_path: str = ""
    benchmark_task_info: BenchmarkTaskInfo = field(default_factory=BenchmarkTaskInfo)
    benchmark_result: BenchmarkResult = field(default_factory=BenchmarkResult)



from typing import Optional

import numpy as np


class ClassificationPostprocessor:
    def __init__(self, top_k_max=20):
        self.top_k_max = top_k_max

    def __call__(self, outputs, k: Optional[int] = None):
        pred = outputs['pred']

        maxk = min(self.top_k_max, pred.shape[1])
        if k:
            maxk = min(k, maxk)

        # Find the indices of the top k values along the second axis (classes)
        pred_indices = np.argsort(-pred, axis=1)[:, :maxk]

        return pred_indices



from typing import Optional

import requests
from requests import Response

from netspresso.exceptions.common import GatewayTimeoutException, UnexpetedException


class Requester:
    @staticmethod
    def __make_response(response: Response) -> Response:
        if response.ok:
            return response
        else:
            try:
                error_message = response.json()
                raise Exception(error_message)
            except ValueError:
                if response.status_code == 504:
                    raise GatewayTimeoutException(error_log=response.text) from None
                else:
                    raise UnexpetedException(error_log=response.text, status_code=response.status_code) from None


    @staticmethod
    def get(url: str, params: Optional[dict] = None, headers=None, **kwargs) -> Response:
        response = requests.get(url, headers=headers, params=params, **kwargs)

        return Requester.__make_response(response=response)

    @staticmethod
    def post_as_form(url: str, request_body: Optional[dict] = None, binary=None, headers=None, **kwargs) -> Response:
        response = requests.post(url, headers=headers, data=request_body, files=binary, **kwargs)

        return Requester.__make_response(response=response)

    @staticmethod
    def post_as_json(url: str, request_body: dict = None, headers=None, **kwargs) -> Response:
        response = requests.post(url, headers=headers, json=request_body, **kwargs)

        return Requester.__make_response(response=response)

    @staticmethod
    def put(url: str, request_body: dict, headers=None, **kwargs) -> Response:
        response = requests.put(url, headers=headers, json=request_body, **kwargs)

        return Requester.__make_response(response=response)

    @staticmethod
    def patch(url: str, request_body: dict, headers=None, **kwargs) -> Response:
        response = requests.patch(url, headers=headers, json=request_body, **kwargs)

        return Requester.__make_response(response=response)

    @staticmethod
    def delete(url: str, headers=None, **kwargs) -> Response:
        response = requests.delete(url, headers=headers, **kwargs)

        return Requester.__make_response(response=response)



import configparser
import os
from pathlib import Path

from dotenv import find_dotenv, load_dotenv
from loguru import logger

from netspresso.enums import EndPointProperty, EnvironmentType, ServiceModule, ServiceName

BASE_DIR = Path(__file__).resolve().parent
config_parser = configparser.ConfigParser()
DEPLOYMENT_MODE = os.getenv("DEPLOYMENT_MODE", "v2-prod-cloud")
config_parser.read(f"{BASE_DIR}/configs/config-{DEPLOYMENT_MODE.lower()}.ini")


class Config:
    _printed = False

    def __init__(self, service_name: ServiceName, module: ServiceModule):
        self.ENVIRONMENT_TYPE = EnvironmentType(DEPLOYMENT_MODE.lower())
        self.SERVICE_NAME = service_name
        self.MODULE = module

        dotenv_path = find_dotenv(filename="netspresso.env")
        if dotenv_path:
            load_dotenv(dotenv_path)

        if self.SERVICE_NAME == ServiceName.TAO:
            self.HOST = config_parser[ServiceName.TAO][EndPointProperty.HOST]
            self.PORT = int(config_parser[ServiceName.TAO][EndPointProperty.PORT])
            self.URI_PREFIX = config_parser[f"TAO.{self.MODULE}"][EndPointProperty.URI_PREFIX]
        else:
            self.HOST = os.environ.get("HOST", config_parser[ServiceName.NP][EndPointProperty.HOST])
            self.PORT = int(os.environ.get("PORT", config_parser[ServiceName.NP][EndPointProperty.PORT]))
            self.URI_PREFIX = config_parser[f"NP.{self.MODULE}"][EndPointProperty.URI_PREFIX]

        self._print_host_and_port()

    def _print_host_and_port(self):
        if not Config._printed and self.SERVICE_NAME == ServiceName.NP:
            logger.info(f"Host: {self.HOST}, Port: {self.PORT}")
            Config._printed = True

    def is_cloud(self) -> bool:
        return self.ENVIRONMENT_TYPE in [EnvironmentType.V2_PROD_CLOUD, EnvironmentType.V2_STAGING_CLOUD, EnvironmentType.V2_DEV_CLOUD]



import os

from loguru import logger

from netspresso.clients.launcher.v2.implements import ConvertTaskAPI, ModelAPI
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    InputLayer,
    RequestConvert,
    RequestModelUploadUrl,
    RequestUploadModel,
    RequestValidateModel,
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    ResponseConvertTaskItem,
    ResponseModelItem,
    ResponseModelOptions,
    ResponseModelUploadUrl,
    UploadDataset,
    UploadFile,
)
from netspresso.clients.utils.common import read_file_bytes
from netspresso.enums import (
    DataType,
    DeviceName,
    Framework,
    LauncherTask,
    SoftwareVersion,
)


class Converter:
    def __init__(self, url):
        self.task_type = LauncherTask.CONVERT.value
        self.convert_task = ConvertTaskAPI(url=url)
        self.convert_model = ModelAPI(url=url, task_type=self.task_type)

    def presigned_model_upload_url(self, access_token: str, input_model_path: str) -> ResponseModelUploadUrl:
        object_name = os.path.basename(input_model_path)

        get_upload_url_request_body = RequestModelUploadUrl(object_name=object_name, task=self.task_type)
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Convert model upload: path - {input_model_path}")

        upload_url_response_body = self.convert_model.get_upload_url(
            request_params=get_upload_url_request_body, headers=token_header
        )
        logger.info(f"Request Convert upload_url result: {upload_url_response_body}")
        return upload_url_response_body

    def upload_model_file(self, access_token: str, input_model_path: str, presigned_upload_url: str) -> str:
        object_name = os.path.basename(input_model_path)
        file_content = read_file_bytes(file_path=input_model_path)

        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(
            f"Request Convert model validation:"
            f" path | {input_model_path} "
            f" presigned_upload_url | {presigned_upload_url}"
        )

        get_model_upload_request_body = RequestUploadModel(url=presigned_upload_url)
        model_file_object = UploadFile(file_name=object_name, file_content=file_content)

        upload_result = self.convert_model.upload(
            request_body=get_model_upload_request_body,
            file=model_file_object,
            headers=token_header,
        )
        logger.info(f"Request Convert upload_model_file result: {upload_result}")
        return upload_result

    def validate_model_file(self, access_token: str, input_model_path: str, ai_model_id: str) -> ResponseModelItem:
        object_name = os.path.basename(input_model_path)
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Convert model validation:" f" path - {input_model_path}" f" ai_model_id - {ai_model_id}")

        get_validate_model_request_body = RequestValidateModel(
            ai_model_id=ai_model_id,
            display_name=object_name,
        )

        validated_model = self.convert_model.validate(
            request_body=get_validate_model_request_body, headers=token_header
        )
        logger.info(f"Request Convert validate_model result: {validated_model}")
        return validated_model

    def download_model_file(self, access_token, convert_task_uuid) -> ResponseConvertDownloadModelUrlItem:
        token_header = AuthorizationHeader(access_token=access_token)
        download_url = self.convert_task.get_download_url(headers=token_header, convert_task_uuid=convert_task_uuid)
        logger.info(f"Request converted model download_url: {download_url}")
        return download_url

    def read_model_task_options(self, access_token, ai_model_id) -> ResponseModelOptions:
        token_header = AuthorizationHeader(access_token=access_token)
        model_task_options = self.convert_model.options(headers=token_header, ai_model_id=ai_model_id)
        logger.info(f"Request model task_options: {model_task_options}")
        return model_task_options

    def start_task(
        self,
        access_token: str,
        input_model_id: str,
        target_device_name: DeviceName,
        target_framework: Framework,
        data_type: DataType = None,
        input_layer: InputLayer = None,
        software_version: SoftwareVersion = None,
        dataset_path: str = None,
    ) -> ResponseConvertTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        request_body = RequestConvert(
            input_model_id=input_model_id,
            target_framework=target_framework,
            target_device_name=target_device_name,
            data_type=data_type,
            input_layer=input_layer,
            software_version=software_version,
        )
        logger.info(f"Request Convert body: {request_body}")

        if dataset_path:
            dataset_filename = os.path.basename(dataset_path)
            file_object = UploadDataset(
                file_name=dataset_filename,
                file_content=read_file_bytes(dataset_path),
            )
        else:
            file_object = None
        convert_task_response = self.convert_task.start(
            request_body=request_body, headers=token_header, file=file_object
        )
        logger.info(f"Request Convert result: {convert_task_response}")
        return convert_task_response

    def cancel_task(self, access_token: str, task_id: str) -> ResponseConvertTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Convert Cancel task_id: {task_id}")

        convert_task_response = self.convert_task.cancel(headers=token_header, task_id=task_id)
        logger.info(f"Request Convert Cancel result: {convert_task_response}")
        return convert_task_response

    def read_task(self, access_token: str, task_id: str) -> ResponseConvertTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Convert Read task_id: {task_id}")

        convert_task_response = self.convert_task.read(headers=token_header, task_id=task_id)
        logger.info(f"Request Convert Task Info: {convert_task_response}")
        return convert_task_response

    def delete_task(self, access_token: str, task_id: str) -> ResponseConvertTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Convert delete task_id: {task_id}")

        convert_task_response = self.convert_task.delete(headers=token_header, task_id=task_id)
        logger.info(f"Request Convert Delete Info: {convert_task_response}")
        return convert_task_response

    def read_status(self, access_token: str, task_id: str) -> ResponseConvertStatusItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Convert read_status task_id: {task_id}")

        convert_task_response = self.convert_task.status(headers=token_header, task_id=task_id)
        logger.info(f"Request Convert Task Status: {convert_task_response}")
        return convert_task_response

    def read_options(self, access_token: str) -> ResponseConvertOptionItems:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info("Request Convert options")

        convert_task_option_response = self.convert_task.options(headers=token_header)
        logger.info(f"Response Convert Task Options: {convert_task_option_response}")
        return convert_task_option_response

    def read_framework_options(self, access_token: str, framework: Framework) -> ResponseConvertFrameworkOptionItems:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info("Request Convert options")

        convert_task_option_response = self.convert_task.options_by_model_framework(
            headers=token_header, model_framework=framework
        )
        logger.info(f"Request Convert Task Options by model_framework: {convert_task_option_response}")
        return convert_task_option_response



from datetime import datetime

import jwt
import pytz
from loguru import logger

from netspresso.clients.auth.response_body import TokenResponse, UserResponse
from netspresso.clients.auth.v2.client import AuthClientV2
from netspresso.clients.config import Config, ServiceModule, ServiceName


class AuthClient:
    def __init__(self, config: Config = Config(ServiceName.NP, ServiceModule.AUTH)):
        """
        Initialize the UserSession.
        """

        self.config = config
        self.api_client = AuthClientV2(config=config)

    def is_cloud(self) -> bool:
        # TODO
        return self.config.is_cloud()

    def login(self, email, password, verify_ssl: bool = True) -> TokenResponse:
        return self.api_client.login(
            email=email, password=password, verify_ssl=verify_ssl
        )

    def get_user_info(self, access_token, verify_ssl: bool = True) -> UserResponse:
        return self.api_client.get_user_info(
            access_token=access_token, verify_ssl=verify_ssl
        )

    def get_credit(self, access_token, verify_ssl: bool = True) -> int:
        return self.api_client.get_credit(
            access_token=access_token, verify_ssl=verify_ssl
        )

    def reissue_token(
        self, access_token, refresh_token, verify_ssl: bool = True
    ) -> TokenResponse:
        return self.api_client.reissue_token(
            access_token=access_token,
            refresh_token=refresh_token,
            verify_ssl=verify_ssl,
        )


class TokenHandler:
    def __init__(self, email, password, verify_ssl: bool = True) -> None:
        self.tokens = auth_client.login(
            email=email, password=password, verify_ssl=verify_ssl
        )
        self.email = email
        self.password = password
        self.verify_ssl = verify_ssl

    def check_jwt_exp(self):
        payload = jwt.decode(
            self.tokens.access_token, options={"verify_signature": False}
        )
        return datetime.now(pytz.utc).timestamp() <= payload["exp"]

    def validate_token(self):
        if not self.check_jwt_exp():
            self.tokens = auth_client.login(
                email=self.email, password=self.password, verify_ssl=self.verify_ssl
            )
            logger.info("The token has expired. the token has been reissued.")


auth_client = AuthClient()



import cv2
import numpy as np
from matplotlib import pyplot as plt

from netspresso.inferencer.visualizers.utils import voc_color_map


class SegmentationVisualizer:
    def __init__(self, class_map, pallete=None, normalized=False, brightness_factor=1.5):
        self.cmap = voc_color_map(N=256, normalized=normalized, brightness_factor=brightness_factor)
        self.class_map = class_map

    def draw(self, image, pred, model_input_shape=None):
        result_images = []
        for _real_gray_image in pred:
            converted_image = self._convert(image, _real_gray_image)
            converted_image = np.transpose(converted_image, (1, 2, 0))
            result_images.append(converted_image)

        return result_images[0]

    def _convert(self, image, gray_image):
        assert len(gray_image.shape) == 2
        size = gray_image.shape
        color_image = np.zeros((3, size[0], size[1]), dtype=np.uint8)

        for label in range(0, len(self.cmap)):
            mask = (label == gray_image)
            color_image[0][mask] = self.cmap[label][0]
            color_image[1][mask] = self.cmap[label][1]
            color_image[2][mask] = self.cmap[label][2]

        # handle void
        mask = (gray_image == 255)
        color_image[0][mask] = color_image[1][mask] = color_image[2][mask] = 255

        return color_image



from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict, List, Optional, Union

from omegaconf import MISSING, MissingMandatoryValue

DEFAULT_IMG_SIZE = 256


@dataclass
class Transform:
    name: str = MISSING


@dataclass
class AugmentationConfig:
    img_size: int = DEFAULT_IMG_SIZE
    train: Optional[List] = None
    inference: Optional[List] = None


@dataclass
class CenterCrop(Transform):
    name: str = 'centercrop'
    size: int = DEFAULT_IMG_SIZE


@dataclass
class ColorJitter(Transform):
    name: str = 'colorjitter'
    brightness: Optional[float] = 0.25
    contrast: Optional[float] = 0.25
    saturation: Optional[float] = 0.25
    hue: Optional[float] = 0.1
    p: Optional[float] = 0.5


@dataclass
class HSVJitter(Transform):
    name: str = "hsvjitter"
    h_mag: int = 5
    s_mag: int = 30
    v_mag: int = 30


@dataclass
class Mixing(Transform):
    name: str = "mixing"
    mixup: Optional[List[float]] = field(default=None)
    cutmix: Optional[List[float]] = field(default=None)
    inplace: bool = False


@dataclass
class MosaicDetection(Transform):
    name: str = "mosaicdetection"
    size: List = field(default_factory=lambda: [DEFAULT_IMG_SIZE, DEFAULT_IMG_SIZE])
    mosaic_prob: float = 1.0
    affine_scale: List = field(default_factory=lambda: [0.5, 1.5])
    degrees: float = 10.0
    translate: float = 0.1
    shear: float = 2.0
    enable_mixup: bool = True
    mixup_prob: float = 1.0
    mixup_scale: List = field(default_factory=lambda: [0.5, 1.5])
    fill: int = 114
    mosaic_off_epoch: int = 10


@dataclass
class Pad(Transform):
    name: str = 'pad'
    size: List = field(default_factory=lambda: [DEFAULT_IMG_SIZE, DEFAULT_IMG_SIZE])
    fill: int = 0


@dataclass
class PoseTopDownAffine(Transform):
    name: str = "posetopdownaffine"
    scale: List = field(default_factory=lambda: [0.75, 1.25])
    scale_prob: float = 1.
    translate: float = 0.1
    translate_prob: float = 1.
    rotation: int = 60
    rotation_prob: float = 1.
    size: List = field(default_factory=lambda: [DEFAULT_IMG_SIZE, DEFAULT_IMG_SIZE])


@dataclass
class RandomCrop(Transform):
    name: str = 'randomcrop'
    size: int = DEFAULT_IMG_SIZE


@dataclass
class RandomErasing(Transform):
    name: str = "randomerasing"
    p: float = 0.5
    scale: List = field(default_factory=lambda: [0.02, 0.33])
    ratio: List = field(default_factory=lambda: [0.3, 3.3])
    value: Optional[int] = 0
    inplace: bool = False


@dataclass
class RandomHorizontalFlip(Transform):
    name: str = 'randomhorizontalflip'
    p: float = 0.5


@dataclass
class RandomResize(Transform):
    name: str = "randomresize"
    base_size: List = field(default_factory=lambda: [256, 256])
    stride: int = 32
    random_range: int = 4
    interpolation: str = "bilinear"


@dataclass
class RandomResizedCrop(Transform):
    name: str = 'randomresizedcrop'
    size: int = DEFAULT_IMG_SIZE
    scale: List = field(default_factory=lambda: [0.08, 1.0])
    ratio: List = field(default_factory=lambda: [0.75, 1.33])
    interpolation: Optional[str] = 'bilinear'


@dataclass
class RandomVerticalFlip(Transform):
    name: str = 'randomverticalflip'
    p: float = 0.5


@dataclass
class Resize(Transform):
    name: str = 'resize'
    size: List = field(default_factory=lambda: [DEFAULT_IMG_SIZE, DEFAULT_IMG_SIZE])
    interpolation: Optional[str] = 'bilinear'
    max_size: Optional[int] =  None
    resize_criteria: Optional[int] =  None


class TrivialAugmentWide(Transform):
    name: str = 'trivialaugmentwide'
    num_magnitude_bins: int = 31
    interpolation: str = 'bilinear'
    fill: Optional[int] = None


@dataclass
class RandomMixup(Transform):
    name: str = 'mixup'
    alpha: float = 0.2
    p: float = 1.0
    inplace: bool = False


@dataclass
class RandomCutmix(Transform):
    name: str = 'cutmix'
    alpha: float = 1.0
    p: float = 1.0
    inplace: bool = False


@dataclass
class ClassificationAugmentationConfig(AugmentationConfig):
    img_size: int = 256
    train: Optional[List] = field(default_factory=lambda: [
        RandomResizedCrop(size=256),
        RandomHorizontalFlip(),
        Mixing(mixup=[0.25, 1.0])
    ])
    inference: Optional[List] = field(default_factory=lambda: [
        Resize(size=[256, 256])
    ])


@dataclass
class SegmentationAugmentationConfig(AugmentationConfig):
    img_size: int = 512
    train: Optional[List] = field(default_factory=lambda: [
        RandomResizedCrop(size=512),
        RandomHorizontalFlip(),
        ColorJitter()
    ])
    inference: Optional[List] = field(default_factory=lambda: [
        Resize(size=[512, 512])
    ])


@dataclass
class DetectionAugmentationConfig(AugmentationConfig):
    img_size: int = 512
    train: Optional[List] = field(default_factory=lambda: [
        Resize(size=[512, 512])
    ])
    inference: Optional[List] = field(default_factory=lambda: [
        Resize(size=[512, 512])
    ])



from enum import Enum


class Optimizer(str, Enum):
    ADADELTA = "Adadelta"
    ADAGRAD = "Adagrad"
    ADAM = "Adam"
    ADAMAX = "Adamax"
    ADAMW = "AdamW"
    RMSPROP = "RMSprop"
    SGD = "SGD"

    @classmethod
    def to_display_name(cls, name: str) -> str:
        name_map = {
            "adadelta": cls.ADADELTA,
            "adagrad": cls.ADAGRAD,
            "adam": cls.ADAM,
            "adamax": cls.ADAMAX,
            "adamw": cls.ADAMW,
            "rmsprop": cls.RMSPROP,
            "sgd": cls.SGD,
        }
        return name_map[name.lower()].value


class Scheduler(str, Enum):
    STEPLR = "StepLR"
    POLYNOMIAL_LR_WITH_WARM_UP = "PolynomialLRWithWarmUp"
    COSINE_ANNEALING_LR_WITH_CUSTOM_WARM_UP = "CosineAnnealingLRWithCustomWarmUp"
    COSINE_ANNEALING_WARM_RESTARTS_WITH_CUSTOM_WARM_UP = "CosineAnnealingWarmRestartsWithCustomWarmUp"

    @classmethod
    def to_display_name(cls, name: str) -> str:
        name_map = {
            "step": cls.STEPLR,
            "poly": cls.POLYNOMIAL_LR_WITH_WARM_UP,
            "cosine_no_sgdr": cls.COSINE_ANNEALING_LR_WITH_CUSTOM_WARM_UP,
            "cosine": cls.COSINE_ANNEALING_WARM_RESTARTS_WITH_CUSTOM_WARM_UP,
        }
        return name_map[name.lower()].value



import dataclasses
from dataclasses import dataclass, field
from typing import List, Optional

from netspresso.clients.launcher.v2.schemas import (
    InputLayer,
    ModelOption,
    ResponseItem,
    ResponseItems,
    TaskInfo,
    TaskOption,
)
from netspresso.clients.launcher.v2.schemas.task.common import TaskStatusInfo
from netspresso.enums import Framework, TaskStatusForDisplay
from netspresso.metadata import benchmarker
from netspresso.metadata.benchmarker import BenchmarkTaskInfo


@dataclass
class BenchmarkResult:
    processor: str
    ram_size: float
    file_size: float
    latency: float
    power_consumption: float
    memory_footprint_cpu: float
    memory_footprint_gpu: float

    def to(self, file_size: float) -> benchmarker.BenchmarkResult:
        benchmark_result = benchmarker.BenchmarkResult()
        benchmark_result.memory_footprint_cpu = self.memory_footprint_cpu
        benchmark_result.memory_footprint_gpu = self.memory_footprint_gpu
        benchmark_result.power_consumption = self.power_consumption
        benchmark_result.ram_size = self.ram_size
        benchmark_result.latency = self.latency
        benchmark_result.file_size = file_size

        return benchmark_result


@dataclass
class BenchmarkEnvironment:
    model_framework: str
    library: list
    cpu: str = ""
    gpu: str = ""


@dataclass
class BenchmarkTask:
    benchmark_task_id: str
    input_model_id: str
    input_layer: InputLayer
    status: TaskStatusForDisplay
    error_log: Optional[dict] = None
    benchmark_task_option: Optional[TaskOption] = None
    benchmark_result: Optional[BenchmarkResult] = None
    benchmark_environment: Optional[BenchmarkEnvironment] = None

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.benchmark_task_option = TaskOption(**self.benchmark_task_option)
        self.benchmark_result = BenchmarkResult(**self.benchmark_result)
        self.benchmark_environment = BenchmarkEnvironment(**self.benchmark_environment)

    def to(self) -> BenchmarkTaskInfo:
        device_info = self.benchmark_task_option.target_device
        benchmark_task_info = BenchmarkTaskInfo()
        benchmark_task_info.framework = self.benchmark_task_option.framework
        benchmark_task_info.display_framework = self.benchmark_task_option.display_framework
        benchmark_task_info.benchmark_task_uuid = self.benchmark_task_id
        benchmark_task_info.device_name = device_info.device_name
        benchmark_task_info.display_device_name = device_info.display_device_name
        benchmark_task_info.display_brand_name = device_info.display_brand_name
        benchmark_task_info.software_version = device_info.software_version
        benchmark_task_info.display_software_version = device_info.display_software_version
        benchmark_task_info.data_type = device_info.data_type
        benchmark_task_info.hardware_type = device_info.hardware_type

        return benchmark_task_info


@dataclass
class ResponseBenchmarkTaskItem(ResponseItem):
    data: Optional[BenchmarkTask] = field(default_factory=dict)

    def __post_init__(self):
        self.data = BenchmarkTask(**self.data)


@dataclass
class BenchmarkOption:
    option_name: str
    display_option: str
    framework: Framework
    device: TaskInfo

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.device = TaskInfo(**self.device)


@dataclass
class ResponseBenchmarkOptionItems(ResponseItems):
    data: List[Optional[BenchmarkOption]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [BenchmarkOption(**item) for item in self.data]


@dataclass
class ResponseBenchmarkStatusItem(ResponseItem):
    data: TaskStatusInfo = field(default_factory=TaskStatusInfo)

    def __post_init__(self):
        self.data = TaskStatusInfo(**self.data)

@dataclass
class ResponseBenchmarkFrameworkOptionItems(ResponseItems):
    data: List[Optional[ModelOption]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [ModelOption(**item) for item in self.data]



import tempfile
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List

import cv2
import numpy as np
from loguru import logger
from netspresso_inference_package.inference.inference_service import InferenceService
from omegaconf import OmegaConf

from netspresso.enums import Runtime, Task
from netspresso.inferencer.postprocessors.classification import ClassificationPostprocessor
from netspresso.inferencer.postprocessors.detection import DetectionPostprocessor
from netspresso.inferencer.postprocessors.segmentation import SegmentationPostprocessor
from netspresso.inferencer.preprocessors.base import Preprocessor
from netspresso.inferencer.visualizers.classification import ClassificationVisualizer
from netspresso.inferencer.visualizers.detection import DetectionVisualizer
from netspresso.inferencer.visualizers.segmentation import SegmentationVisualizer


class BaseInferencer:
    def __init__(self) -> None:
        pass

    def _inference(self, input_model_path: str, dataset_path: str):
        inf_service = InferenceService(model_file_path=input_model_path, dataset_file_path=dataset_path)
        inference_results = inf_service.inference()

        return inference_results

    def transpose_input(self, runtime, input):
        if runtime == Runtime.ONNX:
            input = input.transpose(0, 3, 1, 2)
        elif runtime == Runtime.TFLITE:
            pass

        return input

    def transpose_outputs(self, runtime, outputs):
        if runtime == Runtime.ONNX:
            pass
        elif runtime == Runtime.TFLITE:
            outputs = [np.transpose(index, (0, 3, 1, 2)) for index in outputs] # (b, h, w, c) -> (b, c, h, w)

        return outputs

    def save_numpy_data(self, data):
        with tempfile.NamedTemporaryFile(suffix=".npy", delete=False) as temp_file:
            save_path = temp_file.name
            np.save(save_path, data)

        return save_path

    def save_image(self, image, save_path):
        save_path = Path(save_path)

        if not save_path.parent.exists():
            save_path.parent.mkdir(parents=True, exist_ok=True)
            logger.info(f"The folder has been created. Local Path: {save_path.parent}")

        cv2.imwrite(save_path.as_posix(), cv2.cvtColor(image, cv2.COLOR_RGB2BGR))
        logger.info(f"Result image saved at {save_path}.")

        return save_path.as_posix()


class NPInferencer(BaseInferencer):
    def __init__(self, config_path) -> None:
        super().__init__()
        self.config_path = config_path
        self.runtime_config = OmegaConf.load(config_path).runtime
        self.build_preprocessor()
        self.build_postprocessor()
        self.build_visualizer()

    def build_preprocessor(self):
        self.preprocessor = Preprocessor(self.runtime_config.preprocess)

    def build_postprocessor(self):
        if self.runtime_config.task == Task.IMAGE_CLASSIFICATION:
            self.postprocessor = ClassificationPostprocessor()
        elif self.runtime_config.task == Task.OBJECT_DETECTION:
            params = self.runtime_config.postprocess.params
            del params.class_agnostic
            self.postprocessor = DetectionPostprocessor(**params)
        elif self.runtime_config.task == Task.SEMANTIC_SEGMENTATION:
            self.postprocessor = SegmentationPostprocessor()

    def build_visualizer(self):
        params = self.runtime_config.visualize.params
        if self.runtime_config.task == Task.IMAGE_CLASSIFICATION:
            self.visualizer = ClassificationVisualizer(**params)
        if self.runtime_config.task == Task.OBJECT_DETECTION:
            self.visualizer = DetectionVisualizer(**params)
        if self.runtime_config.task == Task.SEMANTIC_SEGMENTATION:
            self.visualizer = SegmentationVisualizer(**params)

    def inference(self, input_model_path: str, image_path: str, save_path: str):
        suffix = Path(input_model_path).suffix
        runtime = Runtime.get_runtime_by_suffix(suffix)

        # Load image
        img = cv2.imread(image_path)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        img_draw = img.copy()

        # Preprocess image
        img = self.preprocessor(img)
        input_data = self.transpose_input(runtime=runtime, input=img)
        dataset_path = self.save_numpy_data(data=input_data)

        # Inference data
        inference_results = self._inference(input_model_path, dataset_path)
        Path(dataset_path).unlink()

        # Postprocess outputs
        outputs = list(inference_results.values())
        outputs = self.transpose_outputs(runtime, outputs)
        model_input_shape = None

        if self.runtime_config.task == Task.IMAGE_CLASSIFICATION:
            pred = self.postprocessor({"pred": outputs[0]}, k=1)[0]
        elif self.runtime_config.task == Task.OBJECT_DETECTION:
            model_input_shape = img.shape[1:3]
            pred = self.postprocessor({"pred": outputs}, model_input_shape)[0]
        elif self.runtime_config.task == Task.SEMANTIC_SEGMENTATION:
            model_input_shape = img.shape[1:3]
            pred = self.postprocessor({"pred": outputs[0]}, model_input_shape)

        # Draw outputs
        img_draw = self.visualizer.draw(image=img_draw, pred=pred, model_input_shape=model_input_shape)

        self.save_image(image=img_draw, save_path=save_path)

        return img_draw


class CustomInferencer(BaseInferencer):
    def __init__(self) -> None:
        super().__init__()

    def inference(self, input_model_path: str, dataset_path: str):
        suffix = Path(input_model_path).suffix
        runtime = Runtime.get_runtime_by_suffix(suffix)

        inference_results = self._inference(input_model_path, dataset_path)

        outputs = list(inference_results.values())
        outputs = self.transpose_outputs(runtime, outputs)

        return outputs



from enum import Enum

from netspresso.exceptions.inferencer import NotSupportedSuffixException


class Runtime(str, Enum):
    ONNX = "onnx"
    TFLITE = "tflite"

    @classmethod
    def get_runtime_by_suffix(cls, suffix: str) -> str:
        suffix_map = {
            ".onnx": cls.ONNX,
            ".tflite": cls.TFLITE,
        }
        runtime = suffix_map.get(suffix.lower(), None)

        if runtime is None:
            raise NotSupportedSuffixException(available_suffixes=list(suffix_map.keys()), suffix=suffix)

        return runtime



from dataclasses import dataclass, field
from typing import List, Optional

from netspresso.clients.launcher.v2.schemas.common import (
    ModelOption,
    ResponseItem,
    ResponseItems,
    ResponsePaginationItems,
)
from netspresso.clients.launcher.v2.schemas.model import (
    ModelBase,
    ModelStatus,
)


@dataclass
class ModelUploadUrlData:
    """ """

    ai_model_id: str
    presigned_upload_url: str


@dataclass
class ResponseModelUploadUrl(ResponseItem):
    """ """

    data: Optional[ModelUploadUrlData] = field(default_factory=dict)

    def __post_init__(self):
        self.data = ModelUploadUrlData(**self.data)


@dataclass
class ResponseModelStatus(ResponseItem):
    """ """

    data: ModelStatus = field(default_factory=ModelStatus)

    def __post_init__(self):
        self.data = ModelStatus(**self.data)


@dataclass
class ResponseModelItem(ResponseItem):
    """ """

    data: ModelBase = field(default_factory=ModelBase)

    def __post_init__(self):
        self.data = ModelBase(**self.data)


@dataclass
class ResponseModelOptions(ResponseItems):
    """ """

    data: Optional[List[ModelOption]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [ModelOption(**option) for option in self.data]


@dataclass
class ResponseModelItems(ResponsePaginationItems):
    """ """

    data: Optional[List[ModelBase]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [ModelBase(**model) for model in self.data]



from functools import partial

import cv2
import numpy as np

IMAGENET_DEFAULT_MEAN = (0.485, 0.456, 0.406)
IMAGENET_DEFAULT_STD = (0.229, 0.224, 0.225)
INVERSE_MODES_MAPPING = {
    'bilinear': cv2.INTER_LINEAR,
    'bicubic': cv2.INTER_CUBIC,
}


def resize_img(img, size, interpolation, max_size, resize_criteria):
    assert isinstance(size, int), "Only support int type ``size`` now."
    assert resize_criteria == 'long', "Only support ``long`` type ``resize_criteria`` now."
    h, w = img.shape[:2]
    long_side, short_side = max(h, w), min(h, w)
    resize_factor = size / long_side
    target_size = [size, round(resize_factor * short_side)] if h < w else [round(resize_factor * short_side), size]
    img = cv2.resize(img, dsize=target_size, interpolation=cv2.INTER_LINEAR)
    return img


def pad_img(img, size, fill):
    assert isinstance(size, int), "Only support int type ``size`` now."
    h, w = img.shape[:2]
    padded = np.full((size, size, 3), fill, dtype='uint8')
    padded[:h, :w] = img
    return padded


def normalize(img):
    img = img.astype('float32') / 255.0
    img = img - np.array(IMAGENET_DEFAULT_MEAN).reshape(1, 1, -1).astype('float32')
    img = img / np.array(IMAGENET_DEFAULT_STD).reshape(1, 1, -1).astype('float32')
    img = img[np.newaxis, ...]
    return img


class Preprocessor:
    DEVICE_TRANSFORM_DICT = {
        'resize': resize_img,
        'pad': pad_img,
    }

    def __init__(self, preprocess_list):
        self.transforms = []
        for transform in preprocess_list:
            name = transform["name"]
            augment_kwargs = list(transform.keys())
            augment_kwargs.remove('name')
            augment_kwargs = {k:transform[k] for k in augment_kwargs}
            transform = partial(self.DEVICE_TRANSFORM_DICT[name], **augment_kwargs)
            self.transforms.append(transform)

    def __call__(self, img):
        for transform in self.transforms:
            img = transform(img)
        img = normalize(img)
        return img



from .request_body import RequestConvert
from .response_body import (
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    ResponseConvertTaskItem,
)

__all__ = [
    RequestConvert,
    ResponseConvertTaskItem,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems
]



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class MixNetSmallArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mixnet",
            "params": {
                "stem_channels": 16,
                "wid_mul": 1.0,
                "dep_mul": 1.0,
                "dropout_rate": 0.0,
            },
            "stage_params": [
                {
                    "expansion_ratio": [1, 6, 3],
                    "out_channels": [16, 24, 24],
                    "num_blocks": [1, 1, 1],
                    "kernel_sizes": [[3], [3], [3]],
                    "num_exp_groups": [1, 2, 2],
                    "num_poi_groups": [1, 2, 2],
                    "stride": [1, 2, 1],
                    "act_type": ["relu", "relu", "relu"],
                    "se_reduction_ratio": [None, None, None],
                },
                {
                    "expansion_ratio": [6, 6],
                    "out_channels": [40, 40],
                    "num_blocks": [1, 3],
                    "kernel_sizes": [[3, 5, 7], [3, 5]],
                    "num_exp_groups": [1, 2],
                    "num_poi_groups": [1, 2],
                    "stride": [2, 1],
                    "act_type": ["swish", "swish"],
                    "se_reduction_ratio": [2, 2],
                },
                {
                    "expansion_ratio": [6, 6, 6, 3],
                    "out_channels": [80, 80, 120, 120],
                    "num_blocks": [1, 2, 1, 2],
                    "kernel_sizes": [[3, 5, 7], [3, 5], [3, 5, 7], [3, 5, 7, 9]],
                    "num_exp_groups": [1, 1, 2, 2],
                    "num_poi_groups": [2, 2, 2, 2],
                    "stride": [2, 1, 1, 1],
                    "act_type": ["swish", "swish", "swish", "swish"],
                    "se_reduction_ratio": [4, 4, 2, 2],
                },
                {
                    "expansion_ratio": [6, 6],
                    "out_channels": [200, 200],
                    "num_blocks": [1, 2],
                    "kernel_sizes": [[3, 5, 7, 9, 11], [3, 5, 7, 9]],
                    "num_exp_groups": [1, 1],
                    "num_poi_groups": [1, 2],
                    "stride": [2, 1],
                    "act_type": ["swish", "swish"],
                    "se_reduction_ratio": [2, 2],
                },
            ],
        }
    )


@dataclass
class MixNetMediumArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mixnet",
            "params": {
                "stem_channels": 24,
                "wid_mul": 1.0,
                "dep_mul": 1.0,
                "dropout_rate": 0.0,
            },
            "stage_params": [
                {
                    "expansion_ratio": [1, 6, 3],
                    "out_channels": [24, 32, 32],
                    "num_blocks": [1, 1, 1],
                    "kernel_sizes": [[3], [3, 5, 7], [3]],
                    "num_exp_groups": [1, 2, 2],
                    "num_poi_groups": [1, 2, 2],
                    "stride": [1, 2, 1],
                    "act_type": ["relu", "relu", "relu"],
                    "se_reduction_ratio": [None, None, None],
                },
                {
                    "expansion_ratio": [6, 6],
                    "out_channels": [40, 40],
                    "num_blocks": [1, 3],
                    "kernel_sizes": [[3, 5, 7, 9], [3, 5]],
                    "num_exp_groups": [1, 2],
                    "num_poi_groups": [1, 2],
                    "stride": [2, 1],
                    "act_type": ["swish", "swish"],
                    "se_reduction_ratio": [2, 2],
                },
                {
                    "expansion_ratio": [6, 6, 6, 3],
                    "out_channels": [80, 80, 120, 120],
                    "num_blocks": [1, 3, 1, 3],
                    "kernel_sizes": [[3, 5, 7], [3, 5, 7, 9], [3], [3, 5, 7, 9]],
                    "num_exp_groups": [1, 2, 1, 2],
                    "num_poi_groups": [1, 2, 1, 2],
                    "stride": [2, 1, 1, 1],
                    "act_type": ["swish", "swish", "swish", "swish"],
                    "se_reduction_ratio": [4, 4, 2, 2],
                },
                {
                    "expansion_ratio": [6, 6],
                    "out_channels": [200, 200],
                    "num_blocks": [1, 3],
                    "kernel_sizes": [[3, 5, 7, 9], [3, 5, 7, 9]],
                    "num_exp_groups": [1, 1],
                    "num_poi_groups": [1, 2],
                    "stride": [2, 1],
                    "act_type": ["swish", "swish"],
                    "se_reduction_ratio": [2, 2],
                },
            ],
        }
    )


@dataclass
class MixNetLargeArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mixnet",
            "params": {
                "stem_channels": 24,
                "wid_mul": 1.3,
                "dep_mul": 1.0,
                "dropout_rate": 0.0,
            },
            "stage_params": [
                {
                    "expansion_ratio": [1, 6, 3],
                    "out_channels": [24, 32, 32],
                    "num_blocks": [1, 1, 1],
                    "kernel_sizes": [[3], [3, 5, 7], [3]],
                    "num_exp_groups": [1, 2, 2],
                    "num_poi_groups": [1, 2, 2],
                    "stride": [1, 2, 1],
                    "act_type": ["relu", "relu", "relu"],
                    "se_reduction_ratio": [None, None, None],
                },
                {
                    "expansion_ratio": [6, 6],
                    "out_channels": [40, 40],
                    "num_blocks": [1, 3],
                    "kernel_sizes": [[3, 5, 7, 9], [3, 5]],
                    "num_exp_groups": [1, 2],
                    "num_poi_groups": [1, 2],
                    "stride": [2, 1],
                    "act_type": ["swish", "swish"],
                    "se_reduction_ratio": [2, 2],
                },
                {
                    "expansion_ratio": [6, 6, 6, 3],
                    "out_channels": [80, 80, 120, 120],
                    "num_blocks": [1, 3, 1, 3],
                    "kernel_sizes": [[3, 5, 7], [3, 5, 7, 9], [3], [3, 5, 7, 9]],
                    "num_exp_groups": [1, 2, 1, 2],
                    "num_poi_groups": [1, 2, 1, 2],
                    "stride": [2, 1, 1, 1],
                    "act_type": ["swish", "swish", "swish", "swish"],
                    "se_reduction_ratio": [4, 4, 2, 2],
                },
                {
                    "expansion_ratio": [6, 6],
                    "out_channels": [200, 200],
                    "num_blocks": [1, 3],
                    "kernel_sizes": [[3, 5, 7, 9], [3, 5, 7, 9]],
                    "num_exp_groups": [1, 1],
                    "num_poi_groups": [1, 2],
                    "stride": [2, 1],
                    "act_type": ["swish", "swish"],
                    "se_reduction_ratio": [2, 2],
                },
            ],
        }
    )


@dataclass
class ClassificationMixNetSmallModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "mixnet_s"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetSmallArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class SegmentationMixNetSmallModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "mixnet_s"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetSmallArchitectureConfig(
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )


@dataclass
class DetectionMixNetSmallModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "mixnet_s"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetSmallArchitectureConfig(
            neck={
                "name": "fpn",
                "params": {
                    "num_outs": 4,
                    "start_level": 0,
                    "end_level": -1,
                    "add_extra_convs": False,
                    "relu_before_extra_convs": False,
                },
            },
            head={
                "name": "anchor_decoupled_head",
                "params": {
                    # Anchor parameters
                    "anchor_sizes": [
                        [
                            32,
                        ],
                        [
                            64,
                        ],
                        [
                            128,
                        ],
                        [
                            256,
                        ],
                    ],
                    "aspect_ratios": [0.5, 1.0, 2.0],
                    "num_layers": 1,
                    "norm_type": "batch_norm",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "retinanet_loss", "weight": None},
        ]
    )


@dataclass
class ClassificationMixNetMediumModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "mixnet_m"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetMediumArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class SegmentationMixNetMediumModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "mixnet_m"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetMediumArchitectureConfig(
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )


@dataclass
class DetectionMixNetMediumModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "mixnet_m"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetMediumArchitectureConfig(
            neck={
                "name": "fpn",
                "params": {
                    "num_outs": 4,
                    "start_level": 0,
                    "end_level": -1,
                    "add_extra_convs": False,
                    "relu_before_extra_convs": False,
                },
            },
            head={
                "name": "anchor_decoupled_head",
                "params": {
                    # Anchor parameters
                    "anchor_sizes": [
                        [
                            32,
                        ],
                        [
                            64,
                        ],
                        [
                            128,
                        ],
                        [
                            256,
                        ],
                    ],
                    "aspect_ratios": [0.5, 1.0, 2.0],
                    "num_layers": 1,
                    "norm_type": "batch_norm",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "retinanet_loss", "weight": None},
        ]
    )


@dataclass
class ClassificationMixNetLargeModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "mixnet_l"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetLargeArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class SegmentationMixNetLargeModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "mixnet_l"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetLargeArchitectureConfig(
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )


@dataclass
class DetectionMixNetLargeModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "mixnet_l"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MixNetLargeArchitectureConfig(
            neck={
                "name": "fpn",
                "params": {
                    "num_outs": 4,
                    "start_level": 0,
                    "end_level": -1,
                    "add_extra_convs": False,
                    "relu_before_extra_convs": False,
                },
            },
            head={
                "name": "anchor_decoupled_head",
                "params": {
                    # Anchor parameters
                    "anchor_sizes": [
                        [
                            32,
                        ],
                        [
                            64,
                        ],
                        [
                            128,
                        ],
                        [
                            256,
                        ],
                    ],
                    "aspect_ratios": [0.5, 1.0, 2.0],
                    "num_layers": 1,
                    "norm_type": "batch_norm",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "retinanet_loss", "weight": None},
        ]
    )
